# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django import template

# Local application / specific library imports
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model('forum', 'Forum')

PermissionHandler = get_class('forum_permission.handler', 'PermissionHandler')

register = template.Library()


@register.assignment_tag(takes_context=True)
def get_forum_last_post(context, forum, user):
    """
    This will return the last post that can be read by the passed user (permissions check).

    Usage::

        {% get_forum_last_post forum request.user as var %}
    """
    request = context.get('request', None)
    perm_handler = request.forum_permission_handler if request else PermissionHandler()

    # Retrieve the last post link associated with the current forum
    last_post = perm_handler.get_forum_last_post(forum, user)
    return last_post


@register.inclusion_tag('forum/forum_list.html', takes_context=True)
def forum_list(context, forums):
    """
    This will render the given list of forums by respecting the order and the depth of each
    forum in the forums tree.

    Usage::

        {% forum_list my_forums %}
    """
    request = context.get('request', None)
    data_dict = {
        'forums': forums,
        'user': request.user,
        'request': request,
    }

    forums_copy = sorted(forums, key=lambda forum: forum.level)
    if forums_copy:
        root_level = forums_copy[0].level
        data_dict['root_level'] = root_level
        data_dict['root_level_middle'] = root_level + 1
        data_dict['root_level_sub'] = root_level + 2

    return data_dict



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django import template
from django.template.defaultfilters import stringfilter

# Local application / specific library imports
from machina.models.fields import render_func

register = template.Library()


@register.filter(is_safe=True)
@stringfilter
def rendered(value):
    return render_func(value)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.db.models.signals import post_migrate
from django.dispatch import receiver

# Local application / specific library imports
from machina.core.loading import get_class

ForumPermission = get_class('forum_permission.models', 'ForumPermission')
PermissionConfig = get_class('forum_permission.defaults', 'PermissionConfig')


def create_permissions():
    for config in PermissionConfig.permissions:
        try:
            gp = ForumPermission.objects.get(codename=config['fields']['codename'])
        except ForumPermission.DoesNotExist:
            gp = ForumPermission(**config['fields'])
        gp.save()


@receiver(post_migrate)
def create_global_permissions(sender, **kwargs):
    if sender.name.endswith('forum_permission'):
        create_permissions()



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.contrib import admin

# Local application / specific library imports
from machina.core.db.models import get_model

Attachment = get_model('forum_attachments', 'Attachment')


class AttachmentAdmin(admin.ModelAdmin):
    list_display = ('id', 'post', 'comment', 'file', )
    list_display_links = ('id', 'post', 'comment', )
    raw_id_fields = ('post', )


admin.site.register(Attachment, AttachmentAdmin)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class PollsApp(Application):
    name = None

    poll_vote_view = get_class('forum_conversation.forum_polls.views', 'TopicPollVoteView')

    def get_urls(self):
        return [
            url(r'^poll/(?P<pk>\d+)/vote/$', self.poll_vote_view.as_view(), name='topic_poll_vote'),
        ]


application = PollsApp()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import mptt.fields
import machina.models.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Forum',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_active', models.BooleanField(default=True, db_index=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Update date')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('description', machina.models.fields.MarkupTextField(no_rendered_field=True, null=True, verbose_name='Description', blank=True)),
                ('image', machina.models.fields.ExtendedImageField(upload_to='machina/forum_images', null=True, verbose_name='Forum image', blank=True)),
                ('link', models.URLField(null=True, verbose_name='Forum link', blank=True)),
                ('link_redirects', models.BooleanField(default=False, help_text='Records the number of times a forum link was clicked', verbose_name='Track link redirects count')),
                ('type', models.PositiveSmallIntegerField(db_index=True, verbose_name='Forum type', choices=[(0, 'Default forum'), (1, 'Category forum'), (2, 'Link forum')])),
                ('posts_count', models.PositiveIntegerField(default=0, verbose_name='Number of posts', editable=False, blank=True)),
                ('topics_count', models.PositiveIntegerField(default=0, verbose_name='Number of topics', editable=False, blank=True)),
                ('link_redirects_count', models.PositiveIntegerField(default=0, verbose_name='Track link redirects count', editable=False, blank=True)),
                ('last_post_on', models.DateTimeField(null=True, verbose_name='Last post added on', blank=True)),
                ('display_sub_forum_list', models.BooleanField(default=True, help_text='Displays this forum on the legend of its parent-forum (sub forums list)', verbose_name='Display in parent-forums legend')),
                ('_description_rendered', models.TextField(null=True, editable=False, blank=True)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('parent', mptt.fields.TreeForeignKey(related_name='children', verbose_name='Parent', blank=True, to='forum.Forum', null=True)),
            ],
            options={
                'ordering': ['tree_id', 'lft'],
                'abstract': False,
                'verbose_name': 'Forum',
                'verbose_name_plural': 'Forums',
                'permissions': [],
            },
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
from collections import OrderedDict

# Third party imports
from django import forms
from django.conf.urls import url
from django.contrib import admin
from django.contrib.admin import helpers
from django.contrib.admin.widgets import ForeignKeyRawIdWidget
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.urlresolvers import reverse
from django.forms.forms import NON_FIELD_ERRORS
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.shortcuts import render
from django.utils.translation import ugettext_lazy as _
from mptt.forms import TreeNodeChoiceField
from mptt.exceptions import InvalidMove

# Local application / specific library imports
from machina.core.db.models import get_model
from machina.core.loading import get_class
from machina.models.fields import MarkupTextField
from machina.models.fields import MarkupTextFieldWidget

Forum = get_model('forum', 'Forum')
ForumPermission = get_model('forum_permission', 'ForumPermission')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')

PermissionConfig = get_class('forum_permission.defaults', 'PermissionConfig')

PERM_GRANTED = 'granted'
PERM_NOT_GRANTED = 'not-granted'
PERM_NOT_SET = 'not-set'


class ForumAdmin(admin.ModelAdmin):
    """
    The ForumAdmin class provides a specific view for moving up or down any forums.
    """
    formfield_overrides = {
        MarkupTextField: {'widget': MarkupTextFieldWidget},
    }

    fieldsets = (
        [None, {
            'fields': ('type', 'parent', 'name', 'description', 'image',)
        }],
        [_('Forum settings'), {
            'fields': ('display_sub_forum_list',),
            'classes': ('collapse',)
        }],
        [_('Link forum settings'), {
            'fields': ('link', 'link_redirects',),
            'classes': ('collapse',)
        }],
    )
    list_display = ('name', 'type', 'topics_count', 'posts_count',)
    search_fields = ('name',)

    editpermissions_index_view_template_name = 'admin/forum/forum/editpermissions_index.html'
    editpermissions_user_view_template_name = 'admin/forum/forum/editpermissions_user.html'
    editpermissions_anonymous_user_view_template_name = 'admin/forum/forum/editpermissions_anonymous_user.html'
    editpermissions_group_view_template_name = 'admin/forum/forum/editpermissions_group.html'

    def get_urls(self):
        urls = super(ForumAdmin, self).get_urls()
        forum_admin_urls = [
            url(r'^(?P<forum_id>[0-9]+)/move-forum/(?P<direction>up|down)/$',
                self.admin_site.admin_view(self.moveforum_view),
                name='forum_forum_move'),
            url(r'^edit-global-permissions/$',
                self.admin_site.admin_view(self.editpermissions_index_view),
                name='forum_forum_editpermission_index'),
            url(r'^edit-global-permissions/user/(?P<user_id>[0-9]+)/$',
                self.admin_site.admin_view(self.editpermissions_user_view),
                name='forum_forum_editpermission_user'),
            url(r'^edit-global-permissions/user/anonymous/$',
                self.admin_site.admin_view(self.editpermissions_anonymous_user_view),
                name='forum_forum_editpermission_anonymous_user'),
            url(r'^edit-global-permissions/group/(?P<group_id>[0-9]+)/$',
                self.admin_site.admin_view(self.editpermissions_group_view),
                name='forum_forum_editpermission_group'),
            url(r'^(?P<forum_id>[0-9]+)/edit-permissions/$',
                self.admin_site.admin_view(self.editpermissions_index_view),
                name='forum_forum_editpermission_index'),
            url(r'^(?P<forum_id>[0-9]+)/edit-permissions/user/(?P<user_id>[0-9]+)/$',
                self.admin_site.admin_view(self.editpermissions_user_view),
                name='forum_forum_editpermission_user'),
            url(r'^(?P<forum_id>[0-9]+)/edit-permissions/user/anonymous/$',
                self.admin_site.admin_view(self.editpermissions_anonymous_user_view),
                name='forum_forum_editpermission_anonymous_user'),
            url(r'^(?P<forum_id>[0-9]+)/edit-permissions/group/(?P<group_id>[0-9]+)/$',
                self.admin_site.admin_view(self.editpermissions_group_view),
                name='forum_forum_editpermission_group'),
        ]
        return forum_admin_urls + urls

    def get_forum_perms_base_context(self, request, obj=None):
        """
        Returns context dictionary with common admin and object permissions
        related content.
        """
        context = {
            'adminform': {'model_admin': self},
            'media': self.media,
            'object': obj,
            'app_label': self.model._meta.app_label,
            'opts': self.model._meta,
            'has_change_permission': self.has_change_permission(request, obj),
        }
        try:
            context.update(self.admin_site.each_context(request))
        except TypeError:  # pragma: no cover
            # Django 1.7 compatibility
            context.update(self.admin_site.each_context())
        except AttributeError:  # pragma: no cover
            pass
        return context

    def moveforum_view(self, request, forum_id, direction):
        """
        Moves the given forum toward the requested direction.
        """
        forum = get_object_or_404(Forum, pk=forum_id)

        # Fetch the target
        target, position = None, None
        if direction == 'up':
            target, position = forum.get_previous_sibling(), 'left'
        elif direction == 'down':
            target, position = forum.get_next_sibling(), 'right'

        # Do the move
        try:
            assert target is not None
            forum.move_to(target, position)
        except (InvalidMove, AssertionError):
            pass
        self.message_user(request, _("'{}' forum successfully moved").format(forum.name))
        return HttpResponseRedirect(reverse('admin:forum_forum_changelist'))

    def editpermissions_index_view(self, request, forum_id=None):
        """
        Display a form to select a user or a group in order to edit its
        permissions for the considered forum.
        """
        forum = get_object_or_404(Forum, pk=forum_id) if forum_id \
            else None

        # Set up the context
        context = self.get_forum_perms_base_context(request, forum)
        context['forum'] = forum
        context['title'] = _('Forum permissions') if forum \
            else _('Global forum permissions')

        # Handles "copy permission from" form
        permissions_copied = False
        if forum and request.method == 'POST':
            forum_form = PickForumForm(request.POST)
            if forum_form.is_valid() and forum_form.cleaned_data['forum']:
                self._copy_forum_permissions(forum_form.cleaned_data['forum'], forum)
                self.message_user(request, _('Permissions successfully copied'))
                permissions_copied = True
            context['forum_form'] = forum_form
        elif forum:
            context['forum_form'] = PickForumForm()

        # Handles user or group selection
        if request.method == 'POST' and not permissions_copied:
            user_form = PickUserForm(request.POST, admin_site=self.admin_site)
            group_form = PickGroupForm(request.POST, admin_site=self.admin_site)

            if user_form.is_valid() and group_form.is_valid():
                user = user_form.cleaned_data.get('user', None) \
                    if user_form.cleaned_data else None
                anonymous_user = user_form.cleaned_data.get('anonymous_user', None) \
                    if user_form.cleaned_data else None
                group = group_form.cleaned_data.get('group', None) \
                    if group_form.cleaned_data else None

                if not user and not anonymous_user and not group:
                    user_form._errors[NON_FIELD_ERRORS] = user_form.error_class(
                        [_('Choose either a user ID, a group ID or the anonymous user'), ])
                elif user:
                    # Redirect to user
                    url_kwargs = {'forum_id': forum.id, 'user_id': user.id} if forum \
                        else {'user_id': user.id}
                    return redirect(reverse(
                        'admin:forum_forum_editpermission_user', kwargs=url_kwargs))
                elif anonymous_user:
                    # Redirect to anonymous user
                    url_kwargs = {'forum_id': forum.id} if forum else {}
                    return redirect(reverse(
                        'admin:forum_forum_editpermission_anonymous_user', kwargs=url_kwargs))
                elif group:
                    # Redirect to group
                    url_kwargs = {'forum_id': forum.id, 'group_id': group.id} if forum \
                        else {'group_id': group.id}
                    return redirect(reverse(
                        'admin:forum_forum_editpermission_group', kwargs=url_kwargs))

            context['user_errors'] = helpers.AdminErrorList(user_form, [])
            context['group_errors'] = helpers.AdminErrorList(group_form, [])
        else:
            user_form = PickUserForm(admin_site=self.admin_site)
            group_form = PickGroupForm(admin_site=self.admin_site)

        context['user_form'] = user_form
        context['group_form'] = group_form

        return render(
            request, self.editpermissions_index_view_template_name, context)

    def editpermissions_user_view(self, request, user_id, forum_id=None):
        """
        Display a form to define which permissions are granted for the given user
        for the considered forum.
        """
        user_model = get_user_model()
        user = get_object_or_404(user_model, pk=user_id)
        forum = get_object_or_404(Forum, pk=forum_id) if forum_id \
            else None

        # Set up the context
        context = self.get_forum_perms_base_context(request, forum)
        context['forum'] = forum
        context['title'] = '{} - {}'.format(_('Forum permissions'), user)
        context['form'] = self._get_permissions_form(
            request, UserForumPermission, {'forum': forum, 'user': user})

        return render(
            request, self.editpermissions_user_view_template_name, context)

    def editpermissions_anonymous_user_view(self, request, forum_id=None):
        """
        Display a form to define which permissions are granted for the anonymous user
        for the considered forum.
        """
        forum = get_object_or_404(Forum, pk=forum_id) if forum_id \
            else None

        # Set up the context
        context = self.get_forum_perms_base_context(request, forum)
        context['forum'] = forum
        context['title'] = '{} - {}'.format(_('Forum permissions'), _('Anonymous user'))
        context['form'] = self._get_permissions_form(
            request, UserForumPermission, {'forum': forum, 'anonymous_user': True})

        return render(
            request, self.editpermissions_anonymous_user_view_template_name, context)

    def editpermissions_group_view(self, request, group_id, forum_id=None):
        """
        Display a form to define which permissions are granted for the given group
        for the considered forum.
        """
        group = get_object_or_404(Group, pk=group_id)
        forum = get_object_or_404(Forum, pk=forum_id) if forum_id \
            else None

        # Set up the context
        context = self.get_forum_perms_base_context(request, forum)
        context['forum'] = forum
        context['title'] = '{} - {}'.format(_('Forum permissions'), group)
        context['form'] = self._get_permissions_form(
            request, GroupForumPermission, {'forum': forum, 'group': group})

        return render(
            request, self.editpermissions_group_view_template_name, context)

    def _get_permissions_form(self, request, permission_model, filter_kwargs):
        # Fetch the permissions
        forum = filter_kwargs.get('forum', None)
        perm_type_filter = {'is_local': True} if forum else \
            {'is_global': True}
        editable_permissions = ForumPermission.objects.filter(**perm_type_filter) \
            .order_by('name')
        granted_permissions = permission_model.objects.filter(
            permission__in=editable_permissions, has_perm=True, **filter_kwargs) \
            .values_list('permission__codename', flat=True)
        non_granted_permissions = permission_model.objects.filter(
            permission__in=editable_permissions, has_perm=False, **filter_kwargs) \
            .values_list('permission__codename', flat=True)

        permissions_dict = OrderedDict()
        for p in editable_permissions:
            if p.codename in granted_permissions:
                perm_state = PERM_GRANTED
            elif p.codename in non_granted_permissions:
                perm_state = PERM_NOT_GRANTED
            else:
                perm_state = PERM_NOT_SET
            permissions_dict[p.codename] = (p, perm_state)

        if request.method == 'POST':
            form = PermissionsForm(request.POST, permissions_dict=permissions_dict)
            if form.is_valid():
                for codename, value in form.cleaned_data.items():
                    try:
                        perm = permission_model.objects.get(
                            permission=permissions_dict[codename][0], **filter_kwargs)
                    except permission_model.DoesNotExist:
                        if value == PERM_NOT_SET:
                            continue
                        perm = permission_model.objects.create(
                            permission=permissions_dict[codename][0], **filter_kwargs)

                    if value == PERM_NOT_SET:
                        perm.delete()
                        continue

                    perm.has_perm = (value == PERM_GRANTED)
                    perm.save()

                self.message_user(request, _('Permissions successfully applied'))
        else:
            form = PermissionsForm(permissions_dict=permissions_dict)

        return form

    def _copy_forum_permissions(self, forum_from, forum_to):
        user_perms = UserForumPermission.objects.filter(
            forum=forum_from)
        group_perms = GroupForumPermission.objects.filter(
            forum=forum_from)

        for perm in user_perms:
            try:
                new_perm = UserForumPermission.objects.get(
                    permission=perm.permission, forum=forum_to,
                    user=perm.user, anonymous_user=perm.anonymous_user)
            except UserForumPermission.DoesNotExist:
                new_perm = UserForumPermission(
                    permission=perm.permission, forum=forum_to,
                    user=perm.user, anonymous_user=perm.anonymous_user)
            new_perm.has_perm = perm.has_perm
            new_perm.save()

        for perm in group_perms:
            try:
                new_perm = GroupForumPermission.objects.get(
                    permission=perm.permission, forum=forum_to,
                    group=perm.group)
            except GroupForumPermission.DoesNotExist:
                new_perm = GroupForumPermission(
                    permission=perm.permission, forum=forum_to,
                    group=perm.group)
            new_perm.has_perm = perm.has_perm
            new_perm.save()


class PickUserForm(forms.Form):
    user = UserForumPermission._meta.get_field('user').formfield()
    anonymous_user = forms.BooleanField(
        label=_('Anonymous'),
        initial=False,
        help_text=_('Please select this option if you want to edit the permissions of the anonymous user'))

    def __init__(self, *args, **kwargs):
        admin_site = kwargs.pop('admin_site')
        super(PickUserForm, self).__init__(*args, **kwargs)

        self.fields['user'].required = False
        self.fields['user'].widget = ForeignKeyRawIdWidget(
            UserForumPermission._meta.get_field('user').rel,
            admin_site)

        self.fields['anonymous_user'].required = False

    def clean(self):
        cleaned_data = super(PickUserForm, self).clean()
        user = cleaned_data.get('user', None)
        anonymous_user = cleaned_data.get('anonymous_user', None)
        if user and anonymous_user:
            self._errors[NON_FIELD_ERRORS] = self.error_class(
                [_('Choose either a user ID or select the anonymous user'), ])
        return cleaned_data


class PickGroupForm(forms.Form):
    group = GroupForumPermission._meta.get_field('group').formfield()

    def __init__(self, *args, **kwargs):
        admin_site = kwargs.pop('admin_site')
        super(PickGroupForm, self).__init__(*args, **kwargs)

        self.fields['group'].required = False
        self.fields['group'].widget = ForeignKeyRawIdWidget(
            GroupForumPermission._meta.get_field('group').rel,
            admin_site)


class PickForumForm(forms.Form):
    forum = TreeNodeChoiceField(
        queryset=Forum.objects.all(), required=False)


class PermissionsForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self.permissions_dict = kwargs.pop('permissions_dict', {})
        super(PermissionsForm, self).__init__(*args, **kwargs)

        # Initializes permission fields
        f_choices = (
            (PERM_NOT_SET, _('Not set')),
            (PERM_GRANTED, _('Granted')),
            (PERM_NOT_GRANTED, _('Not granted')),
        )
        for scope in PermissionConfig.scopes:
            codenames = [x['fields']['codename'] for x in PermissionConfig.permissions
                         if x['scope'] == scope]
            permissions = filter(lambda v: v[0] in codenames, self.permissions_dict.items())
            for codename, p in permissions:
                self.fields[codename] = forms.ChoiceField(
                    label=p[0].name,
                    choices=f_choices,
                    required=False,
                    widget=forms.RadioSelect)
                self.fields[codename].initial = p[1]
                self.fields[codename].scope = scope


admin.site.register(Forum, ForumAdmin)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class ModerationApp(Application):
    name = 'forum_moderation'

    topic_lock_view = get_class('forum_moderation.views', 'TopicLockView')
    topic_unlock_view = get_class('forum_moderation.views', 'TopicUnlockView')
    topic_delete_view = get_class('forum_moderation.views', 'TopicDeleteView')
    topic_move_view = get_class('forum_moderation.views', 'TopicMoveView')
    topic_update_to_normal_topic_view = get_class('forum_moderation.views', 'TopicUpdateToNormalTopicView')
    topic_update_to_sticky_topic_view = get_class('forum_moderation.views', 'TopicUpdateToStickyTopicView')
    topic_update_to_announce_view = get_class('forum_moderation.views', 'TopicUpdateToAnnounceView')
    moderation_queue_list_view = get_class('forum_moderation.views', 'ModerationQueueListView')
    moderation_queue_detail_view = get_class('forum_moderation.views', 'ModerationQueueDetailView')
    post_approve_view = get_class('forum_moderation.views', 'PostApproveView')
    post_disapprove_view = get_class('forum_moderation.views', 'PostDisapproveView')

    def get_urls(self):
        return [
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/lock/$'), self.topic_lock_view.as_view(), name='topic_lock'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/unlock/$'), self.topic_unlock_view.as_view(), name='topic_unlock'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/delete/$'), self.topic_delete_view.as_view(), name='topic_delete'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/move/$'), self.topic_move_view.as_view(), name='topic_move'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/change/topic/$'),
                self.topic_update_to_normal_topic_view.as_view(), name='topic_update_to_post'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/change/sticky/$'),
                self.topic_update_to_sticky_topic_view.as_view(), name='topic_update_to_sticky'),
            url(_(r'^topic/(?P<slug>[\w-]+)-(?P<pk>\d+)/change/announce/$'),
                self.topic_update_to_announce_view.as_view(), name='topic_update_to_announce'),
            url(_(r'^queue/$'), self.moderation_queue_list_view.as_view(), name='queue'),
            url(_(r'^queue/(?P<pk>\d+)/$'), self.moderation_queue_detail_view.as_view(), name='queued_post'),
            url(_(r'^queue/(?P<pk>\d+)/approve/$'), self.post_approve_view.as_view(), name='approve_queued_post'),
            url(_(r'^queue/(?P<pk>\d+)/disapprove/$'), self.post_disapprove_view.as_view(), name='disapprove_queued_post'),
        ]


application = ModerationApp()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.loading import get_class

ForumReadTrackManager = get_class('forum_tracking.managers', 'ForumReadTrackManager')


@python_2_unicode_compatible
class AbstractForumReadTrack(models.Model):
    """
    Represents a track which records which forums have been read by a given user.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='forum_tracks', verbose_name=_('User'))
    forum = models.ForeignKey('forum.Forum', verbose_name=_('Forum'), related_name='tracks')
    mark_time = models.DateTimeField(auto_now=True)

    objects = ForumReadTrackManager()

    class Meta:
        abstract = True
        app_label = 'forum_tracking'
        unique_together = ['user', 'forum', ]
        verbose_name = _('Forum track')
        verbose_name_plural = _('Forum tracks')

    def __str__(self):
        return '{} - {}'.format(self.user, self.forum)


@python_2_unicode_compatible
class AbstractTopicReadTrack(models.Model):
    """
    Represents a track which records which topics have been read by a given user.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='topic_tracks', verbose_name=_('User'))
    topic = models.ForeignKey('forum_conversation.Topic', verbose_name=_('Topic'), related_name='tracks')
    mark_time = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True
        app_label = 'forum_tracking'
        unique_together = ['user', 'topic', ]
        verbose_name = _('Topic track')
        verbose_name_plural = _('Topic tracks')

    def __str__(self):
        return '{} - {}'.format(self.user, self.topic)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.forms.forms import NON_FIELD_ERRORS
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import UpdateView
from django.views.generic.edit import ModelFormMixin

# Local application / specific library imports
from machina.core.db.models import get_model
from machina.core.loading import get_class

TopicPoll = get_model('forum_polls', 'TopicPoll')
TopicPollVote = get_model('forum_polls', 'TopicPollVote')

TopicPollVoteForm = get_class('forum_polls.forms', 'TopicPollVoteForm')

PermissionRequiredMixin = get_class('forum_permission.viewmixins', 'PermissionRequiredMixin')


class TopicPollVoteView(PermissionRequiredMixin, UpdateView):
    """
    Allows to vote in polls.
    """
    model = TopicPoll
    form_class = TopicPollVoteForm
    http_method_names = ['post', ]

    def get_form_kwargs(self):
        kwargs = super(ModelFormMixin, self).get_form_kwargs()
        kwargs['poll'] = self.object
        return kwargs

    def form_valid(self, form):
        user_kwargs = {'voter': self.request.user} if self.request.user.is_authenticated() \
            else {'anonymous_key': self.request.user.forum_key}

        if self.object.user_changes:
            # If user changes are allowed for this poll, all
            # the poll associated with the current user must
            # be deleted.
            TopicPollVote.objects.filter(
                poll_option__poll=self.object,
                **user_kwargs).delete()

        options = form.cleaned_data['options']
        for option in options:
            TopicPollVote.objects.create(
                poll_option=option, **user_kwargs)

        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form):
        messages.error(self.request, form.errors[NON_FIELD_ERRORS])
        return redirect(reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.topic.forum.slug,
            'forum_pk': self.object.topic.forum.pk,
            'slug': self.object.topic.slug,
            'pk': self.object.topic.pk}))

    def get_success_url(self):
        messages.success(self.request, _('Your vote has been cast.'))
        return reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.topic.forum.slug,
            'forum_pk': self.object.topic.forum.pk,
            'slug': self.object.topic.slug,
            'pk': self.object.topic.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object()

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_vote_in_poll(obj, user)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf import settings

# Local application / specific library imports


# General
MACHINA_FORUM_NAME = getattr(settings, 'MACHINA_FORUM_NAME', 'Machina')
MACHINA_MARKUP_LANGUAGE = getattr(settings, 'MACHINA_MARKUP_LANGUAGE', ('django_markdown.utils.markdown', {}))
MACHINA_MARKUP_WIDGET = getattr(settings, 'MACHINA_MARKUP_WIDGET', 'django_markdown.widgets.MarkdownWidget')


# Forum
FORUM_IMAGE_UPLOAD_TO = getattr(settings, 'MACHINA_FORUM_IMAGE_UPLOAD_TO', 'machina/forum_images')
FORUM_IMAGE_WIDTH = getattr(settings, 'MACHINA_FORUM_IMAGE_WIDTH', 100)
FORUM_IMAGE_HEIGHT = getattr(settings, 'MACHINA_FORUM_IMAGE_HEIGHT', 70)

DEFAULT_FORUM_IMAGE_SETTINGS = {
    'width': FORUM_IMAGE_WIDTH,
    'height': FORUM_IMAGE_HEIGHT
}

FORUM_TOPICS_NUMBER_PER_PAGE = getattr(settings, 'MACHINA_FORUM_TOPICS_NUMBER_PER_PAGE', 20)


# Conversation
TOPIC_ANSWER_SUBJECT_PREFIX = getattr(settings, 'MACHINA_TOPIC_ANSWER_SUBJECT_PREFIX', 'Re:')

TOPIC_POSTS_NUMBER_PER_PAGE = getattr(settings, 'MACHINA_TOPIC_POSTS_NUMBER_PER_PAGE', 15)
TOPIC_REVIEW_POSTS_NUMBER = getattr(settings, 'MACHINA_TOPIC_REVIEW_POSTS_NUMBER', 10)


# Polls
POLL_MAX_OPTIONS_PER_POLL = getattr(settings, 'MACHINA_POLL_MAX_OPTIONS_PER_POLL', 30)
POLL_MAX_OPTIONS_PER_USER = getattr(settings, 'MACHINA_POLL_MAX_OPTIONS_PER_USER', 10)


# Attachments
ATTACHMENT_FILE_UPLOAD_TO = getattr(settings, 'MACHINA_ATTACHMENT_FILE_UPLOAD_TO', 'machina/attachments')
ATTACHMENT_CACHE_NAME = getattr(settings, 'MACHINA_ATTACHMENT_CACHE_NAME', 'machina_attachments')
ATTACHMENT_MAX_FILES_PER_POST = getattr(settings, 'MACHINA_ATTACHMENT_MAX_FILES_PER_POST', 15)

# Member
PROFILE_AVATAR_UPLOAD_TO = getattr(settings, 'MACHINA_PROFILE_AVATAR_UPLOAD_TO', 'machina/avatar_images')

PROFILE_AVATAR_WIDTH = getattr(settings, 'MACHINA_PROFILE_AVATAR_WIDTH', 150)
PROFILE_AVATAR_HEIGHT = getattr(settings, 'MACHINA_PROFILE_AVATAR_HEIGHT', 250)
PROFILE_AVATAR_MIN_WIDTH = getattr(settings, 'MACHINA_PROFILE_AVATAR_MIN_WIDTH', None)
PROFILE_AVATAR_MAX_WIDTH = getattr(settings, 'MACHINA_PROFILE_AVATAR_MAX_WIDTH', None)
PROFILE_AVATAR_MIN_HEIGHT = getattr(settings, 'MACHINA_PROFILE_AVATAR_MIN_HEIGHT', None)
PROFILE_AVATAR_MAX_HEIGHT = getattr(settings, 'MACHINA_PROFILE_AVATAR_MAX_HEIGHT', None)
PROFILE_AVATAR_MAX_UPLOAD_SIZE = getattr(settings, 'MACHINA_PROFILE_AVATAR_MAX_UPLOAD_SIZE', 0)

DEFAULT_AVATAR_SETTINGS = {
    'width': PROFILE_AVATAR_WIDTH,
    'height': PROFILE_AVATAR_HEIGHT,
    'min_width': PROFILE_AVATAR_MIN_WIDTH,
    'max_width': PROFILE_AVATAR_MAX_WIDTH,
    'min_height': PROFILE_AVATAR_MIN_HEIGHT,
    'max_height': PROFILE_AVATAR_MAX_HEIGHT,
    'max_upload_size': PROFILE_AVATAR_MAX_UPLOAD_SIZE
}

PROFILE_SIGNATURE_MAX_LENGTH = getattr(settings, 'MACHINA_PROFILE_SIGNATURE_MAX_LENGTH', 255)

PROFILE_RECENT_POSTS_NUMBER = getattr(settings, 'MACHINA_PROFILE_RECENT_POSTS_NUMBER', 15)


# Permission
DEFAULT_AUTHENTICATED_USER_FORUM_PERMISSIONS = getattr(
    settings, 'MACHINA_DEFAULT_AUTHENTICATED_USER_FORUM_PERMISSIONS', [])



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.contrib import admin

# Local application / specific library imports
from machina.core.db.models import get_model

ForumPermission = get_model('forum_permission', 'ForumPermission')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')


class ForumPermissionAdmin(admin.ModelAdmin):
    search_fields = ('name', 'codename',)
    list_display = ('name', 'codename', 'is_global', 'is_local',)
    list_editables = ('is_global', 'is_local',)


class GroupForumPermissionAdmin(admin.ModelAdmin):
    search_fields = ('permission__name', 'permission__codename', 'group__name',)
    list_display = ('group', 'permission', 'has_perm', )
    list_editables = ('has_perm',)
    raw_id_fields = ('group',)


class UserForumPermissionAdmin(admin.ModelAdmin):
    search_fields = ('permission__name', 'permission__codename', 'user__username',)
    list_display = ('user', 'anonymous_user', 'permission', 'has_perm', )
    list_editables = ('has_perm',)
    raw_id_fields = ('user',)


admin.site.register(ForumPermission, ForumPermissionAdmin)
admin.site.register(GroupForumPermission, GroupForumPermissionAdmin)
admin.site.register(UserForumPermission, UserForumPermissionAdmin)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
import django.dispatch

# Local application / specific library imports


topic_viewed = django.dispatch.Signal(providing_args=["topic", "user", "request", "response", ])



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.generic import ListView

# Local application / specific library imports
from machina.apps.forum.signals import forum_viewed
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model('forum', 'Forum')
Topic = get_model('forum_conversation', 'Topic')

PermissionRequiredMixin = get_class('forum_permission.viewmixins', 'PermissionRequiredMixin')


class IndexView(ListView):
    """
    Displays the top-level forums.
    """
    template_name = 'forum/index.html'
    context_object_name = 'forums'

    def get_queryset(self):
        return self.request.forum_permission_handler.forum_list_filter(
            Forum.objects.displayable_subforums(),
            self.request.user
        )

    def get_context_data(self, **kwargs):
        context = super(IndexView, self).get_context_data(**kwargs)

        top_level_forums = context['forums'].filter(parent__isnull=True)
        context['total_posts_count'] = sum(f.posts_count for f in top_level_forums)
        context['total_topics_count'] = sum(f.topics_count for f in top_level_forums)

        return context


class ForumView(PermissionRequiredMixin, ListView):
    """
    Displays a forums and its topics. If applicable, its sub-forums can
    also be displayed.
    """
    template_name = 'forum/forum_detail.html'
    context_object_name = 'topics'
    permission_required = ['can_read_forum', ]
    paginate_by = machina_settings.FORUM_TOPICS_NUMBER_PER_PAGE
    view_signal = forum_viewed

    def get(self, request, **kwargs):
        forum = self.get_forum()
        if forum.is_link:
            response = HttpResponseRedirect(forum.link)
        else:
            response = super(ForumView, self).get(request, **kwargs)
        self.send_signal(request, response, forum)
        return response

    def get_forum(self):
        """
        Returns the forum to consider.
        """
        if not hasattr(self, 'forum'):
            self.forum = get_object_or_404(Forum, pk=self.kwargs['pk'])
        return self.forum

    def get_queryset(self):
        self.forum = self.get_forum()
        qs = self.forum.topics.exclude(type=Topic.TYPE_CHOICES.topic_announce).exclude(approved=False) \
            .select_related('poster')
        return qs

    def get_controlled_object(self):
        return self.get_forum()

    def get_context_data(self, **kwargs):
        context = super(ForumView, self).get_context_data(**kwargs)

        # Insert the considered forum into the context
        context['forum'] = self.get_forum()

        # Get the list of forums that have the current forum as parent
        sub_forums = Forum.objects.displayable_subforums(start_from=self.forum)
        context['sub_forums'] = self.request.forum_permission_handler \
            .forum_list_filter(sub_forums, self.request.user)

        # The announces will be displayed on each page of the forum
        context['announces'] = self.get_forum().topics.filter(type=Topic.TYPE_CHOICES.topic_announce)

        return context

    def send_signal(self, request, response, forum):
        self.view_signal.send(
            sender=self, forum=forum, user=request.user,
            request=request, response=response)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from haystack.views import search_view_factory

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class SearchApp(Application):
    name = 'forum_search'

    search_view = get_class('forum_search.views', 'FacetedSearchView')
    search_form = get_class('forum_search.forms', 'SearchForm')

    def get_urls(self):
        return [
            url(r'^$', search_view_factory(
                view_class=self.search_view,
                form_class=self.search_form),
                name='search'),
        ]


application = SearchApp()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals
from os import path

# Third party imports
from django.core.exceptions import ImproperlyConfigured
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db import models
from django.db.models import signals
from django.forms import ValidationError
from django.template.defaultfilters import filesizeformat
from django.utils.encoding import python_2_unicode_compatible
from django.utils.encoding import smart_str
from django.utils.functional import curry
from django.utils.safestring import mark_safe
from django.utils.safestring import SafeData
from django.utils.six import BytesIO
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.conf import settings as machina_settings


_rendered_field_name = lambda name: '_{}_rendered'.format(name)


def _get_markup_widget():
    dotted_path = machina_settings.MACHINA_MARKUP_WIDGET
    try:
        module, widget = dotted_path.rsplit('.', 1)
        module, widget = smart_str(module), smart_str(widget)
        widget = getattr(__import__(module, {}, {}, [widget]), widget)
        return widget
    except ImportError as e:
        raise ImproperlyConfigured(_('Could not import MACHINA_MARKUP_WIDGET {}: {}').format(
            machina_settings.MACHINA_MARKUP_WIDGET,
            e))

MarkupTextFieldWidget = _get_markup_widget()


def _get_render_function(dotted_path, kwargs):
    module, func = dotted_path.rsplit('.', 1)
    module, func = smart_str(module), smart_str(func)
    func = getattr(__import__(module, {}, {}, [func]), func)
    return curry(func, **kwargs)


try:
    markup_lang = machina_settings.MACHINA_MARKUP_LANGUAGE
    render_func = _get_render_function(markup_lang[0], markup_lang[1]) if markup_lang \
        else lambda text: text
except ImportError as e:
    raise ImproperlyConfigured(_('Could not import MACHINA_MARKUP_LANGUAGE {}: {}').format(
        machina_settings.MACHINA_MARKUP_LANGUAGE,
        e))
except AttributeError as e:
    raise ImproperlyConfigured(_('MACHINA_MARKUP_LANGUAGE setting is required'))


@python_2_unicode_compatible
class MarkupText(SafeData):
    def __init__(self, instance, field_name, rendered_field_name):
        # Stores a reference to the instance along with field names
        # to make assignment possible.
        self.instance = instance
        self.field_name = field_name
        self.rendered_field_name = rendered_field_name

    # raw is read/write
    def _get_raw(self):
        return self.instance.__dict__[self.field_name]

    def _set_raw(self, val):
        setattr(self.instance, self.field_name, val)

    raw = property(_get_raw, _set_raw)

    # rendered is a read only property
    def _get_rendered(self):
        return mark_safe(getattr(self.instance, self.rendered_field_name))

    rendered = property(_get_rendered)

    # Allows display via templates to work without safe filter
    def __str__(self):
        return self.raw

    # Return the length of the rendered string so that bool tests work as expected
    def __len__(self):
        return len(self.raw)


class MarkupTextDescriptor(object):
    """
    Acts as the Django's default attribute descriptor class (enabled via the SubfieldBase metaclass).
    The main difference is that it does not call to_python() on the MarkupTextField class. Instead, it
    stores the two different values of a markup content (the raw and the rendered data) separately.
    These values can be separately updated when something is assigned. When the field is accessed,
    a MarkupText instance will be returned ; this one is built with the current data.
    """
    def __init__(self, field):
        self.field = field
        self.rendered_field_name = _rendered_field_name(self.field.name)

    def __get__(self, instance, owner):
        if instance is None:
            raise AttributeError(_('Can only be accessed via an instance.'))
        raw = instance.__dict__[self.field.name]
        if raw is None:
            return None
        return MarkupText(instance, self.field.name, self.rendered_field_name)

    def __set__(self, instance, value):
        if isinstance(value, MarkupText):
            instance.__dict__[self.field.name] = value.raw
            setattr(instance, self.rendered_field_name, value.rendered)
        else:
            # Set only the raw field
            instance.__dict__[self.field.name] = value


class MarkupTextField(models.TextField):
    """
    A MarkupTextField contributes two columns to the model instead of the standard single column.
    The initial column store any content written by using a given markup language and the other one
    keeps the rendered content returned by a specific render function.
    """
    def __init__(self, *args, **kwargs):
        # For Django 1.7 migration serializer compatibility: the frozen version of a
        # MarkupTextField can't try to add a '*_rendered' field, because the '*_rendered' field itself
        # is frozen / serialized as well.
        self.add_rendered_field = not kwargs.pop('no_rendered_field', False)
        super(MarkupTextField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        """
        As outlined in the Django 1.7 documentation, this method tells Django how to take an instance
        of a new field in order to reduce it to a serialized form. This can be used to configure what
        arguments need to be passed to the __init__() method of the field in order to re-create it.
        We use it in order to pass the 'no_rendered_field' to the __init__() method. This will allow
        the _rendered field to not be added to the model class twice.
        """
        name, import_path, args, kwargs = super(MarkupTextField, self).deconstruct()
        kwargs['no_rendered_field'] = True
        return name, import_path, args, kwargs

    def contribute_to_class(self, cls, name):
        if self.add_rendered_field and not cls._meta.abstract:
            rendered_field = models.TextField(editable=False, blank=True, null=True)
            cls.add_to_class(_rendered_field_name(name), rendered_field)

        # The data will be rendered before each save
        signals.pre_save.connect(self.render_data, sender=cls)

        # Add the default text field
        super(MarkupTextField, self).contribute_to_class(cls, name)

        # Associates the name of this field to a special descriptor that will return
        # an appropriate Markup object each time the field is accessed
        setattr(cls, name, MarkupTextDescriptor(self))

    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return value.raw

    def get_db_prep_value(self, value, connection=None, prepared=False):
        try:
            return value.raw
        except AttributeError:
            return value

    def render_data(self, signal, sender, instance=None, **kwargs):
        value = getattr(instance, self.attname)

        rendered = None
        if hasattr(value, 'raw'):
            rendered = render_func(value.raw)

        setattr(instance, _rendered_field_name(self.attname), rendered)

    def formfield(self, **kwargs):
        if machina_settings.MACHINA_MARKUP_WIDGET:
            widget = _get_markup_widget()
            defaults = {'widget': widget}
            defaults.update(kwargs)
        else:
            defaults = kwargs

        field = super(MarkupTextField, self).formfield(**defaults)
        return field


class ExtendedImageField(models.ImageField):
    """
    An ExtendedImageField is an ImageField whose image can be resized before being saved.
    This field also add the capability of checking the image size, width and height a user may send.
    """
    def __init__(self, *args, **kwargs):
        self.width = kwargs.pop('width', None)
        self.height = kwargs.pop('height', None)
        # Both min_width and max_width must be provided in order to be used
        self.min_width = kwargs.pop('min_width', None)
        self.max_width = kwargs.pop('max_width', None)
        # Both min_height and max_height must be provided in order to be used
        self.min_height = kwargs.pop('min_height', None)
        self.max_height = kwargs.pop('max_height', None)
        self.max_upload_size = kwargs.pop('max_upload_size', 0)
        super(ExtendedImageField, self).__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        from django.core.files.images import get_image_dimensions
        data = super(ExtendedImageField, self).clean(*args, **kwargs)
        image = data.file

        # Controls the file size
        if self.max_upload_size and hasattr(image, 'size'):
            if image.size > self.max_upload_size:
                raise ValidationError(
                    _('Files of size greater than {} are not allowed. Your file is {}').format(
                        filesizeformat(self.max_upload_size),
                        filesizeformat(image.size)
                    )
                )

        # Controls the image size
        image_width, image_height = get_image_dimensions(data)
        if (self.min_width and self.max_width and not self.min_width <= image_width <= self.max_width):
            raise ValidationError(
                _('Images of width lesser than {}px or greater than {}px or are not allowed. The width of your image is {}px').format(
                    self.min_width, self.max_width, image_width))
        if self.min_height and self.max_height and not self.min_height <= image_height <= self.max_height:
            raise ValidationError(
                _('Images of height lesser than {}px or greater than {}px or are not allowed. The height of your image is {}px').format(
                    self.min_height, self.max_height, image_height))

        return data

    def save_form_data(self, instance, data):
        if data and self.width and self.height:
            content = self.resize_image(data.read(), (self.width, self.height))

            # Handle the filename because the image will be converted to PNG
            filename = path.splitext(path.split(data.name)[-1])[0]
            filename = '{}.png'.format(filename)

            # Regenerate a File object
            data = SimpleUploadedFile(filename, content)
        super(ExtendedImageField, self).save_form_data(instance, data)

    def resize_image(self, data, size):
        """
        Resizes the given image to fit inside a box of the given size
        """
        from machina.core.compat import PILImage as Image
        image = Image.open(BytesIO(data))

        # Resize!
        image.thumbnail(size, Image.ANTIALIAS)

        string = BytesIO()
        image.save(string, format='PNG')
        return string.getvalue()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class TrackingApp(Application):
    name = 'forum_tracking'

    mark_forums_read_view = get_class('forum_tracking.views', 'MarkForumsReadView')
    mark_topics_read_view = get_class('forum_tracking.views', 'MarkTopicsReadView')
    unread_topics_view = get_class('forum_tracking.views', 'UnreadTopicsView')

    def get_urls(self):
        return [
            url(_(r'^mark/forums/$'), self.mark_forums_read_view.as_view(), name='mark_all_forums_read'),
            url(_(r'^mark/forums/(?P<pk>\d+)/$'), self.mark_forums_read_view.as_view(), name='mark_subforums_read'),
            url(_(r'^mark/forum/(?P<pk>\d+)/topics/$'), self.mark_topics_read_view.as_view(), name='mark_topics_read'),
            url(_(r'^unread-topics/$'), self.unread_topics_view.as_view(), name='unread_topics'),
        ]


application = TrackingApp()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals
import uuid

# Third party imports
# Local application / specific library imports
from machina.core.loading import get_class

PermissionHandler = get_class('forum_permission.handler', 'PermissionHandler')


class ForumPermissionMiddleware(object):
    """
    This middleware attaches an instance of the PermissionHandler to each request.
    This allows to cache the permissions for the lifetime of the request object.
    The middleware also attaches a random identifier to each anonymous user in order
    to perform proper permission checks for anonymous users. This identifier is
    stored in the session.
    """
    anonymous_forum_key_session_id = '_anonymous_forum_key'

    def process_request(self, request):
        if not request.user.is_authenticated():
            # Get the anonymous forum key and attaches it the AnonymousUser instance.
            anonymous_forum_key = request.session.get(self.anonymous_forum_key_session_id, None)
            if anonymous_forum_key is None:
                anonymous_forum_key = self.get_anonymous_forum_key()
                request.session[self.anonymous_forum_key_session_id] = anonymous_forum_key

            setattr(request.user, 'forum_key', anonymous_forum_key)

        request.forum_permission_handler = PermissionHandler()

    def get_anonymous_forum_key(self):
        """
        Returns a random anonymous forum key.
        """
        return uuid.uuid4().hex



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('forum', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('forum_conversation', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ForumReadTrack',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('mark_time', models.DateTimeField(auto_now=True)),
                ('forum', models.ForeignKey(related_name='tracks', verbose_name='Forum', to='forum.Forum')),
                ('user', models.ForeignKey(related_name='forum_tracks', verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Forum track',
                'verbose_name_plural': 'Forum tracks',
            },
        ),
        migrations.CreateModel(
            name='TopicReadTrack',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('mark_time', models.DateTimeField(auto_now=True)),
                ('topic', models.ForeignKey(related_name='tracks', verbose_name='Topic', to='forum_conversation.Topic')),
                ('user', models.ForeignKey(related_name='topic_tracks', verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Topic track',
                'verbose_name_plural': 'Topic tracks',
            },
        ),
        migrations.AlterUniqueTogether(
            name='topicreadtrack',
            unique_together=set([('user', 'topic')]),
        ),
        migrations.AlterUniqueTogether(
            name='forumreadtrack',
            unique_together=set([('user', 'forum')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import machina.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('forum', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Update date')),
                ('poster_ip', models.GenericIPAddressField(default='2002::0', null=True, verbose_name='Poster IP address', blank=True)),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('content', machina.models.fields.MarkupTextField(no_rendered_field=True, verbose_name='Content')),
                ('username', models.CharField(max_length=155, null=True, verbose_name='Username', blank=True)),
                ('approved', models.BooleanField(default=True, verbose_name='Approved')),
                ('update_reason', models.CharField(max_length=255, null=True, verbose_name='Update reason', blank=True)),
                ('updates_count', models.PositiveIntegerField(default=0, verbose_name='Updates count', editable=False, blank=True)),
                ('_content_rendered', models.TextField(null=True, editable=False, blank=True)),
                ('poster', models.ForeignKey(related_name='posts', verbose_name='Poster', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'ordering': ['created'],
                'abstract': False,
                'get_latest_by': 'created',
                'verbose_name': 'Post',
                'verbose_name_plural': 'Posts',
            },
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Update date')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('slug', models.SlugField(max_length=300, verbose_name='Slug')),
                ('type', models.PositiveSmallIntegerField(db_index=True, verbose_name='Topic type', choices=[(0, 'Default topic'), (1, 'Sticky'), (2, 'Announce')])),
                ('status', models.PositiveIntegerField(db_index=True, verbose_name='Topic status', choices=[(0, 'Topic unlocked'), (1, 'Topic locked'), (2, 'Topic moved')])),
                ('approved', models.BooleanField(default=True, verbose_name='Approved')),
                ('posts_count', models.PositiveIntegerField(default=0, verbose_name='Posts count', editable=False, blank=True)),
                ('views_count', models.PositiveIntegerField(default=0, verbose_name='Views count', editable=False, blank=True)),
                ('last_post_on', models.DateTimeField(null=True, verbose_name='Last post added on', blank=True)),
                ('forum', models.ForeignKey(related_name='topics', verbose_name='Topic forum', to='forum.Forum')),
                ('poster', models.ForeignKey(verbose_name='Poster', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('subscribers', models.ManyToManyField(related_name='subscriptions', verbose_name='Subscribers', to=settings.AUTH_USER_MODEL, blank=True)),
            ],
            options={
                'ordering': ['-type', '-last_post_on'],
                'abstract': False,
                'get_latest_by': 'last_post_on',
                'verbose_name': 'Topic',
                'verbose_name_plural': 'Topics',
            },
        ),
        migrations.AddField(
            model_name='post',
            name='topic',
            field=models.ForeignKey(related_name='posts', verbose_name='Topic', to='forum_conversation.Topic'),
        ),
        migrations.AddField(
            model_name='post',
            name='updated_by',
            field=models.ForeignKey(blank=True, editable=False, to=settings.AUTH_USER_MODEL, null=True, verbose_name='Lastly updated by'),
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.shortcuts import _get_queryset

# Local application / specific library imports


def get_object_or_none(klass, *args, **kwargs):
    """
    Calls get() on a given model manager, but it returns None instead of the model’s DoesNotExist exception.
    """
    queryset = _get_queryset(klass)
    try:
        return queryset.get(*args, **kwargs)
    except:
        return None


def refresh(instance):
    """
    Select and return instance from database.
    Usage: instance = refresh(instance)
    """
    return instance.__class__.objects.get(pk=instance.pk)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django import forms
from django.forms.models import BaseModelFormSet
from django.forms.models import modelformset_factory

# Local application / specific library imports
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model

Attachment = get_model('forum_attachments', 'Attachment')


class AttachmentForm(forms.ModelForm):
    class Meta:
        model = Attachment
        fields = ['file', 'comment', ]


class BaseAttachmentFormset(BaseModelFormSet):
    def __init__(self, *args, **kwargs):
        self.post = kwargs.pop('post', None)
        super(BaseAttachmentFormset, self).__init__(*args, **kwargs)

    def save(self, commit=True, **kwargs):
        if self.post:
            for form in self.forms:
                form.instance.post = self.post
        super(BaseAttachmentFormset, self).save(commit)


AttachmentFormset = modelformset_factory(
    Attachment, AttachmentForm,
    formset=BaseAttachmentFormset,
    can_delete=True, extra=1,
    max_num=machina_settings.ATTACHMENT_MAX_FILES_PER_POST,
    validate_max=True)



# -*- coding: utf-8 -*-

# Standard library imports
import mimetypes
import os

# Third party imports
from django.http import HttpResponse
from django.views.generic import DetailView

# Local application / specific library imports
from machina.core.db.models import get_model
from machina.core.loading import get_class

Attachment = get_model('forum_attachments', 'Attachment')

PermissionRequiredMixin = get_class('forum_permission.viewmixins', 'PermissionRequiredMixin')


class AttachmentView(PermissionRequiredMixin, DetailView):
    """
    Allows to retrieve a forum attachment.
    """
    model = Attachment

    def render_to_response(self, context, **response_kwargs):
        filename = os.path.basename(self.object.file.name)

        # Try to guess the content type of the given file
        content_type, _ = mimetypes.guess_type(self.object.file.name)
        if not content_type:
            content_type = 'text/plain'

        response = HttpResponse(self.object.file, content_type=content_type)
        response['Content-Disposition'] = 'attachment; filename={}'.format(filename)

        return response

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().post.topic.forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_download_files(obj, user)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports


default_app_config = 'machina.apps.forum_member.registry_config.MemberRegistryConfig'



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports


class Application(object):
    """
    Machina's views and URLs use a tree of 'app' instances, each of which subclass this Application
    class and provide an 'urls' property.
    This process is directly inspired from the one provided by django-oscar.
    """
    name = None

    def __init__(self, app_name=None, **kwargs):
        self.app_name = app_name
        # Set all kwargs as object attributes
        for key, value in kwargs.items():  # pragma: no cover
            setattr(self, key, value)

    def get_urls(self):
        """
        Return the url patterns for the considered app. It should be implemented in
        any subclass.
        """
        return []

    @property
    def urls(self):
        # We set the application and instance namespaces here
        return self.get_urls(), self.app_name, self.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('forum_conversation', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TopicPoll',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Update date')),
                ('question', models.CharField(max_length=255, verbose_name='Poll question')),
                ('duration', models.PositiveIntegerField(null=True, verbose_name='Poll duration, in days', blank=True)),
                ('max_options', models.PositiveSmallIntegerField(default=1, verbose_name='Maximum number of poll options per user', validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(10)])),
                ('user_changes', models.BooleanField(default=False, verbose_name='Allow vote changes')),
                ('topic', models.OneToOneField(related_name='poll', verbose_name='Topic', to='forum_conversation.Topic')),
            ],
            options={
                'ordering': ['-updated'],
                'abstract': False,
                'get_latest_by': 'updated',
                'verbose_name': 'Topic poll',
                'verbose_name_plural': 'Topic polls',
            },
        ),
        migrations.CreateModel(
            name='TopicPollOption',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('text', models.CharField(max_length=255, verbose_name='Poll option text')),
                ('poll', models.ForeignKey(related_name='options', verbose_name='Poll', to='forum_polls.TopicPoll')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Topic poll option',
                'verbose_name_plural': 'Topic poll options',
            },
        ),
        migrations.CreateModel(
            name='TopicPollVote',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('timestamp', models.DateTimeField(auto_now_add=True, verbose_name="Vote's date")),
                ('poll_option', models.ForeignKey(related_name='votes', verbose_name='Poll option', to='forum_polls.TopicPollOption')),
                ('voter', models.ForeignKey(related_name='poll_votes', verbose_name='Voter', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Topic poll vote',
                'verbose_name_plural': 'Topic poll votes',
            },
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth.models import Group

# Local application / specific library imports
from machina.core.db.models import get_model

ForumPermission = get_model('forum_permission', 'ForumPermission')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')


class NotUserNorGroup(Exception):
    pass


def assign_perm(perm, user_or_group, forum=None, has_perm=True):
    """
    Assign a permission to a user (anonymous or not) or a group.
    """
    user, group = get_identity(user_or_group)
    perm = ForumPermission.objects.get(codename=perm)
    if user:
        return UserForumPermission.objects.create(
            forum=forum,
            permission=perm,
            user=user if not user.is_anonymous() else None,
            anonymous_user=user.is_anonymous(),
            has_perm=has_perm)
    if group:
        return GroupForumPermission.objects.create(
            forum=forum, permission=perm, group=group, has_perm=has_perm)


def remove_perm(perm, user_or_group, forum=None):
    """
    Remove a permission to a user (anonymous or not) or a group.
    """
    user, group = get_identity(user_or_group)
    perm = ForumPermission.objects.get(codename=perm)
    if user:
        UserForumPermission.objects.filter(
            forum=forum,
            permission=perm,
            user=user if not user.is_anonymous() else None,
            anonymous_user=user.is_anonymous()).delete()
    if group:
        GroupForumPermission.objects.filter(
            forum=forum, permission=perm, group=group).delete()


def get_identity(identity):
    """
    Returns a (user_obj, None) tuple or a (None, group_obj) tuple depending on the considered instance.
    """
    if isinstance(identity, AnonymousUser):
        return identity, None

    if isinstance(identity, get_user_model()):
        return identity, None
    elif isinstance(identity, Group):
        return None, identity
    else:  # pragma: no cover
        raise NotUserNorGroup(
            'User/AnonymousUser or Group instance is required '
            '(got {})'.format(identity))


def get_anonymous_user_forum_key(user):
    """
    Returns the forum key identifier associated with the considered anonymous user.
    """
    return user.forum_key if isinstance(user, AnonymousUser) and hasattr(user, 'forum_key') \
        else None



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.db import models
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports


class ActiveManager(models.Manager):
    """
    Returns only active objects.
    """
    def get_queryset(self):
        super_self = super(ActiveManager, self)
        get_queryset = (super_self.get_query_set
                        if hasattr(super_self, 'get_query_set')
                        else super_self.get_queryset)

        return get_queryset().filter(is_active__exact=True)


class ActiveModel(models.Model):
    """
    An abstract base class model that provides an is_active field and attach an ActiveManager.
    """
    is_active = models.BooleanField(default=True, db_index=True)

    # Managers
    objects = models.Manager()
    active = ActiveManager()

    class Meta:
        abstract = True


class DatedModel(models.Model):
    """
    An abstract base class model that provides a created and a updated fields to store creation date and last updated date.
    """
    created = models.DateTimeField(auto_now_add=True, verbose_name=_('Creation date'))
    updated = models.DateTimeField(auto_now=True, verbose_name=_('Update date'))

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('forum', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='forum',
            options={'ordering': ['tree_id', 'lft'], 'verbose_name': 'Forum', 'verbose_name_plural': 'Forums'},
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model('forum', 'Forum')
Topic = get_model('forum_conversation', 'Topic')

PermissionHandler = get_class('forum_permission.handler', 'PermissionHandler')
perm_handler = PermissionHandler()


class LastTopicsFeed(Feed):
    """
    Provides feed items for the latest forum topics.
    """
    # Standard RSS elements
    title = _('Latest topics')
    description = _('Latest topics updated on the forums')
    link = reverse_lazy('forum:index')

    # Item elements
    title_template = 'forum_feeds/topics_title.html'
    description_template = 'forum_feeds/topics_description.html'

    def get_object(self, request, *args, **kwargs):
        forum_pk = kwargs.get('forum_pk', None)
        descendants = kwargs.get('descendants', None)
        self.user = request.user

        if forum_pk:
            forum = get_object_or_404(Forum, pk=forum_pk)
            forums_qs = forum.get_descendants(include_self=True) if descendants \
                else Forum.objects.filter(pk=forum_pk)
            self.forums = perm_handler.forum_list_filter(
                forums_qs, request.user)
        else:
            self.forums = perm_handler.forum_list_filter(
                Forum.objects.all(), request.user)

    def items(self):
        return Topic.objects.filter(forum__in=self.forums, approved=True).order_by('-last_post_on')

    def item_pubdate(self, item):
        return item.created



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import force_text
from django.utils.encoding import python_2_unicode_compatible
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from model_utils import Choices
from mptt.models import MPTTModel
from mptt.models import TreeForeignKey

# Local application / specific library imports
from machina.apps.forum import signals
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from machina.core.loading import get_class
from machina.core.shortcuts import refresh
from machina.models import ActiveModel
from machina.models import DatedModel
from machina.models.fields import ExtendedImageField
from machina.models.fields import MarkupTextField

ForumManager = get_class('forum.managers', 'ForumManager')


FORUM_TYPES = Choices(
    (0, 'forum_post', _('Default forum')),
    (1, 'forum_cat', _('Category forum')),
    (2, 'forum_link', _('Link forum')),
)


@python_2_unicode_compatible
class AbstractForum(MPTTModel, ActiveModel, DatedModel):
    """
    The main forum model.
    The tree hierarchy of forums and categories is managed by the MPTTModel
    which is part of django-mptt.
    """
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children', verbose_name=_('Parent'))

    name = models.CharField(max_length=100, verbose_name=_('Name'))
    slug = models.SlugField(max_length=255, verbose_name=_('Slug'))

    description = MarkupTextField(
        verbose_name=_('Description'),
        null=True, blank=True)

    # A forum can come with an image (eg. a small logo)
    image = ExtendedImageField(verbose_name=_('Forum image'), null=True, blank=True,
                               upload_to=machina_settings.FORUM_IMAGE_UPLOAD_TO,
                               **machina_settings.DEFAULT_FORUM_IMAGE_SETTINGS)

    # Forums can be simple links (eg. wiki, documentation, etc)
    link = models.URLField(verbose_name=_('Forum link'), null=True, blank=True)
    link_redirects = models.BooleanField(verbose_name=_('Track link redirects count'),
                                         help_text=_('Records the number of times a forum link was clicked'),
                                         default=False)

    # Category, Default forum or Link ; that's what a forum can be
    TYPE_CHOICES = FORUM_TYPES
    type = models.PositiveSmallIntegerField(choices=TYPE_CHOICES, verbose_name=_('Forum type'), db_index=True)

    # Tracking data (only approved topics and posts are recorded)
    posts_count = models.PositiveIntegerField(verbose_name=_('Number of posts'), editable=False, blank=True, default=0)
    topics_count = models.PositiveIntegerField(verbose_name=_('Number of topics'), editable=False, blank=True, default=0)
    link_redirects_count = models.PositiveIntegerField(verbose_name=_('Track link redirects count'),
                                                       editable=False, blank=True, default=0)
    last_post_on = models.DateTimeField(verbose_name=_('Last post added on'), blank=True, null=True)

    # Display options
    display_sub_forum_list = models.BooleanField(verbose_name=_('Display in parent-forums legend'),
                                                 help_text=_('Displays this forum on the legend of its parent-forum (sub forums list)'),
                                                 default=True)

    objects = ForumManager()

    class Meta:
        abstract = True
        app_label = 'forum'
        ordering = ['tree_id', 'lft']
        verbose_name = _('Forum')
        verbose_name_plural = _('Forums')

    def __str__(self):
        return self.name

    @property
    def margin_level(self):
        """
        Used in templates or menus to create an easy-to-see left margin to contrast
        a forum from their parents.
        """
        return self.level * 2

    @property
    def is_category(self):
        """
        Returns True if the forum is a category.
        """
        return self.type == FORUM_TYPES.forum_cat

    @property
    def is_forum(self):
        """
        Returns True if the forum is a a default forum.
        """
        return self.type == FORUM_TYPES.forum_post

    @property
    def is_link(self):
        """
        Returns True if the forum is a link.
        """
        return self.type == FORUM_TYPES.forum_link

    def clean(self):
        super(AbstractForum, self).clean()

        if self.parent and self.parent.is_link:
                raise ValidationError(_('A forum can not have a link forum as parent'))

        if self.is_category and self.parent and self.parent.is_category:
                raise ValidationError(_('A category can not have another category as parent'))

        if self.is_link and not self.link:
            raise ValidationError(_('A link forum must have a link associated with it'))

    def save(self, *args, **kwargs):
        # It is vital to track the changes of the parent associated with a forum in order to
        # maintain counters up-to-date and to trigger other operations such as permissions updates.
        old_instance = None
        if self.pk:
            old_instance = self.__class__._default_manager.get(pk=self.pk)

        # Update the slug field
        self.slug = slugify(force_text(self.name))

        # Do the save
        super(AbstractForum, self).save(*args, **kwargs)

        # If any change has been made to the forum parent, trigger the update of the counters
        if old_instance and old_instance.parent != self.parent:
            self.update_trackers()
            # The previous parent trackers should also be updated
            if old_instance.parent:
                old_parent = refresh(old_instance.parent)
                old_parent.update_trackers()
            # Trigger the 'forum_moved' signal
            signals.forum_moved.send(sender=self, previous_parent=old_instance.parent)

    def _simple_save(self, *args, **kwargs):
        """
        Calls the parent save method in order to avoid the checks for forum parent changes
        which can result in triggering a new update of the counters associated with the
        current forum.
        This allow the database to not be hit by such checks during very common and regular
        operations such as those provided by the update_trackers function; indeed these operations
        will never result in an update of a forum parent.
        """
        super(AbstractForum, self).save(*args, **kwargs)

    def update_trackers(self):
        # Fetch the list of ids of all descendant forums including the current one
        forum_ids = self.get_descendants(include_self=True).values_list('id', flat=True)

        # Determine the list of the associated topics, that is the list of topics
        # associated with the current forum plus the list of all topics associated
        # with the descendant forums.
        topic_klass = get_model('forum_conversation', 'Topic')
        topics = topic_klass.objects.filter(forum__id__in=forum_ids).order_by('-last_post_on')
        approved_topics = topics.filter(approved=True)

        self.topics_count = approved_topics.count()
        # Compute the forum level posts count (only approved posts are recorded)
        posts_count = sum(topic.posts_count for topic in topics)
        self.posts_count = posts_count

        # Force the forum 'last_post_on' date to the one associated with the topic with
        # the latest post.
        self.last_post_on = approved_topics[0].last_post_on if len(approved_topics) else None

        # Any save of a forum triggered from the update_tracker process will not result
        # in checking for a change of the forum's parent.
        self._simple_save()

        # Trigger the parent trackers update if necessary
        if self.parent:
            self.parent.update_trackers()

    def get_absolute_url(self):
        from django.core.urlresolvers import reverse
        return reverse('forum:forum', kwargs={'slug': self.slug, 'pk': str(self.id)})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('forum', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ForumPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('codename', models.CharField(unique=True, max_length=150, verbose_name='Permission codename')),
                ('name', models.CharField(max_length=255, null=True, verbose_name='Permission name', blank=True)),
                ('is_global', models.BooleanField(default=False, help_text='This permission can be granted globally to all the forums', verbose_name='Global permission')),
                ('is_local', models.BooleanField(default=True, help_text='This permission can be granted individually for each forum', verbose_name='Local permission')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Forum permission',
                'verbose_name_plural': 'Forum permissions',
            },
        ),
        migrations.CreateModel(
            name='GroupForumPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('has_perm', models.BooleanField(default=True, verbose_name='Has perm')),
                ('forum', models.ForeignKey(verbose_name='Forum', blank=True, to='forum.Forum', null=True)),
                ('group', models.ForeignKey(verbose_name='Group', to='auth.Group')),
                ('permission', models.ForeignKey(verbose_name='Forum permission', to='forum_permission.ForumPermission')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Group forum permission',
                'verbose_name_plural': 'Group forum permissions',
            },
        ),
        migrations.CreateModel(
            name='UserForumPermission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('has_perm', models.BooleanField(default=True, verbose_name='Has perm')),
                ('anonymous_user', models.BooleanField(default=False, verbose_name='Target anonymous user')),
                ('forum', models.ForeignKey(verbose_name='Forum', blank=True, to='forum.Forum', null=True)),
                ('permission', models.ForeignKey(verbose_name='Forum permission', to='forum_permission.ForumPermission')),
                ('user', models.ForeignKey(verbose_name='User', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'User forum permission',
                'verbose_name_plural': 'User forum permissions',
            },
        ),
        migrations.AlterUniqueTogether(
            name='userforumpermission',
            unique_together=set([('permission', 'forum', 'user')]),
        ),
        migrations.AlterUniqueTogether(
            name='groupforumpermission',
            unique_together=set([('permission', 'forum', 'group')]),
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django import forms

# Local application / specific library imports
from machina.core.db.models import get_model

ForumProfile = get_model('forum_member', 'ForumProfile')


class ForumProfileForm(forms.ModelForm):
    class Meta:
        model = ForumProfile
        fields = ['avatar', 'signature', ]



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class ForumApp(Application):
    name = 'forum'

    index_view = get_class('forum.views', 'IndexView')
    forum_view = get_class('forum.views', 'ForumView')

    def get_urls(self):
        return [
            url(r'^$', self.index_view.as_view(), name='index'),
            url(_(r'^forum/(?P<slug>[\w-]+)-(?P<pk>\d+)/$'), self.forum_view.as_view(), name='forum'),
        ]


application = ForumApp()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.contrib import messages
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import DeleteView
from django.views.generic import DetailView
from django.views.generic import ListView
from django.views.generic.detail import BaseDetailView
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.detail import SingleObjectTemplateResponseMixin
from django.views.generic.edit import FormMixin
from django.views.generic.edit import ProcessFormView

# Local application / specific library imports
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model('forum', 'Forum')
Post = get_model('forum_conversation', 'Post')
Topic = get_model('forum_conversation', 'Topic')

TopicMoveForm = get_class('forum_moderation.forms', 'TopicMoveForm')

PermissionRequiredMixin = get_class('forum_permission.viewmixins', 'PermissionRequiredMixin')


class TopicLockView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin, BaseDetailView):
    """
    A view providing the ability to lock forum topics.
    """
    template_name = 'forum_moderation/topic_lock.html'
    context_object_name = 'topic'
    success_message = _('This topic has been locked successfully.')
    model = Topic

    def lock(self, request, *args, **kwargs):
        """
        Locks the considered topic and retirects the user to the success URL.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.status = Topic.STATUS_CHOICES.topic_locked
        self.object.save()
        return HttpResponseRedirect(success_url)

    def post(self, request, *args, **kwargs):
        return self.lock(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(TopicLockView, self).get_context_data(**kwargs)

        # Append the forum associated with the topic being locked
        # to the context
        topic = self.get_object()
        context['forum'] = topic.forum

        return context

    def get_success_url(self):
        messages.success(self.request, self.success_message)

        return reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.forum.slug,
            'forum_pk': self.object.forum.pk,
            'slug': self.object.slug,
            'pk': self.object.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_lock_topics(obj, user)


class TopicUnlockView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin, BaseDetailView):
    """
    A view providing the ability to unlock forum topics.
    """
    template_name = 'forum_moderation/topic_unlock.html'
    context_object_name = 'topic'
    success_message = _('This topic has been unlocked successfully.')
    model = Topic

    def unlock(self, request, *args, **kwargs):
        """
        Unlocks the considered topic and retirects the user to the success URL.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.status = Topic.STATUS_CHOICES.topic_unlocked
        self.object.save()
        return HttpResponseRedirect(success_url)

    def post(self, request, *args, **kwargs):
        return self.unlock(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(TopicUnlockView, self).get_context_data(**kwargs)

        # Append the forum associated with the topic being locked
        # to the context
        topic = self.get_object()
        context['forum'] = topic.forum

        return context

    def get_success_url(self):
        messages.success(self.request, self.success_message)

        return reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.forum.slug,
            'forum_pk': self.object.forum.pk,
            'slug': self.object.slug,
            'pk': self.object.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_lock_topics(obj, user)


class TopicDeleteView(PermissionRequiredMixin, DeleteView):
    """
    A view providing the ability to delete forum topics.
    """
    template_name = 'forum_moderation/topic_delete.html'
    context_object_name = 'topic'
    success_message = _('This topic has been deleted successfully.')
    model = Topic

    def get_context_data(self, **kwargs):
        context = super(TopicDeleteView, self).get_context_data(**kwargs)

        # Append the forum associated with the topic being deleted
        # to the context
        topic = self.get_object()
        context['forum'] = topic.forum

        return context

    def delete(self, request, *args, **kwargs):
        """
        Deletes the considered topic and retirects the user to the success URL.
        Calls the delete() method on the fetched object and then
        redirects to the success URL.
        This is a workaround for versions of Django prior 1.6
        where the get_success_url() method was called after
        the delete() method.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.delete()
        return HttpResponseRedirect(success_url)

    def get_success_url(self):
        messages.success(self.request, self.success_message)

        return reverse('forum:forum', kwargs={
            'slug': self.object.forum.slug,
            'pk': self.object.forum.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_delete_topics(obj, user)


class TopicMoveView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin,
                    FormMixin, SingleObjectMixin, ProcessFormView):
    """
    A view providing the ability to move forum topics.
    """
    template_name = 'forum_moderation/topic_move.html'
    form_class = TopicMoveForm
    context_object_name = 'topic'
    success_message = _('This topic has been moved successfully.')
    model = Topic

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TopicMoveView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(TopicMoveView, self).post(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(TopicMoveView, self).get_context_data(**kwargs)

        # Append the forum associated with the topic being deleted
        # to the context
        topic = self.get_object()
        context['forum'] = topic.forum

        return context

    def get_form_kwargs(self):
        kwargs = super(TopicMoveView, self).get_form_kwargs()
        kwargs.update({
            'topic': self.object,
            'user': self.request.user,
        })
        return kwargs

    def form_valid(self, form):
        # Move the topic
        topic = self.object
        old_forum = topic.forum
        new_forum = form.cleaned_data['forum']
        topic.forum = new_forum

        # Eventually lock the topic
        if form.cleaned_data['lock_topic']:
            topic.status = Topic.STATUS_CHOICES.topic_locked
        else:
            topic.status = Topic.STATUS_CHOICES.topic_moved

        topic.save()
        old_forum.save()

        messages.success(self.request, self.success_message)
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.forum.slug,
            'forum_pk': self.object.forum.pk,
            'slug': self.object.slug,
            'pk': self.object.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_move_topics(obj, user)


class TopicUpdateTypeBaseView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin, BaseDetailView):
    """
    A view providing the ability to change the type of forum topics: normal, sticky topic or announce.
    """
    template_name = 'forum_moderation/topic_update_type.html'
    context_object_name = 'topic'
    model = Topic
    success_message = _('This topic type has been changed successfully.')

    # The following attributes should be defined in subclasses
    target_type = None
    question = ''

    def update_type(self, request, *args, **kwargs):
        """
        Updates the type of the considered topic and retirects the user to the success URL.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.type = self.target_type
        self.object.save()
        return HttpResponseRedirect(success_url)

    def post(self, request, *args, **kwargs):
        return self.update_type(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(TopicUpdateTypeBaseView, self).get_context_data(**kwargs)
        context['question'] = self.question

        # Append the forum associated with the topic being locked
        # to the context
        topic = self.get_object()
        context['forum'] = topic.forum

        return context

    def get_success_url(self):
        messages.success(self.request, self.success_message)

        return reverse('forum_conversation:topic', kwargs={
            'forum_slug': self.object.forum.slug,
            'forum_pk': self.object.forum.pk,
            'slug': self.object.slug,
            'pk': self.object.pk})

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().forum


class TopicUpdateToNormalTopicView(TopicUpdateTypeBaseView):
    target_type = Topic.TYPE_CHOICES.topic_post
    question = _('Would you want to change this topic to a default topic?')

    # Permissions checks

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_update_topics_to_normal_topics(obj, user)


class TopicUpdateToStickyTopicView(TopicUpdateTypeBaseView):
    target_type = Topic.TYPE_CHOICES.topic_sticky
    question = _('Would you want to change this topic to a sticky topic?')

    # Permissions checks

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_update_topics_to_sticky_topics(obj, user)


class TopicUpdateToAnnounceView(TopicUpdateTypeBaseView):
    target_type = Topic.TYPE_CHOICES.topic_announce
    question = _('Would you want to change this topic to an announce?')

    # Permissions checks

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_update_topics_to_announces(obj, user)


class ModerationQueueListView(PermissionRequiredMixin, ListView):
    template_name = 'forum_moderation/moderation_queue/list.html'
    context_object_name = 'posts'
    paginate_by = machina_settings.TOPIC_POSTS_NUMBER_PER_PAGE
    model = Post

    def get_queryset(self):
        forums = self.request.forum_permission_handler.get_moderation_queue_forums(
            self.request.user)
        qs = super(ModerationQueueListView, self).get_queryset()
        qs = qs.filter(topic__forum__in=forums, approved=False)
        return qs.order_by('-created')

    # Permissions checks

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_access_moderation_queue(user)


class ModerationQueueDetailView(PermissionRequiredMixin, DetailView):
    template_name = 'forum_moderation/moderation_queue/detail.html'
    context_object_name = 'post'
    model = Post

    def get_context_data(self, **kwargs):
        context = super(ModerationQueueDetailView, self).get_context_data(**kwargs)

        post = self.object
        topic = post.topic

        # Handles the case when a poll is associated to the topic
        try:
            if hasattr(topic, 'poll') and topic.poll.options.exists():
                poll = topic.poll
                context['poll'] = poll
                context['poll_options'] = poll.options.all()
        except ObjectDoesNotExist:  # pragma: no cover
            pass

        if not post.is_topic_head:
            # Add the topic review
            previous_posts = topic.posts.filter(approved=True, created__lte=post.created).order_by('-created')
            previous_posts = previous_posts[:machina_settings.TOPIC_REVIEW_POSTS_NUMBER]
            context['previous_posts'] = previous_posts

        return context

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().topic.forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_approve_posts(obj, user)


class PostApproveView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin, BaseDetailView):
    """
    A view providing the ability to approve queued forum posts.
    """
    template_name = 'forum_moderation/moderation_queue/post_approve.html'
    context_object_name = 'post'
    success_message = _('This post has been approved successfully.')
    model = Post

    def approve(self, request, *args, **kwargs):
        """
        Approves the considered post and retirects the user to the success URL.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.approved = True
        self.object.save()
        return HttpResponseRedirect(success_url)

    def post(self, request, *args, **kwargs):
        return self.approve(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PostApproveView, self).get_context_data(**kwargs)
        context['forum'] = self.get_object().topic.forum
        return context

    def get_success_url(self):
        messages.success(self.request, self.success_message)
        return reverse('forum_moderation:queue')

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().topic.forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_approve_posts(obj, user)


class PostDisapproveView(PermissionRequiredMixin, SingleObjectTemplateResponseMixin, BaseDetailView):
    """
    A view providing the ability to disapprove queued forum posts.
    """
    template_name = 'forum_moderation/moderation_queue/post_disapprove.html'
    context_object_name = 'post'
    success_message = _('This post has been disapproved successfully.')
    model = Post

    def disapprove(self, request, *args, **kwargs):
        """
        Disapproves the considered post and retirects the user to the success URL.
        """
        self.object = self.get_object()
        success_url = self.get_success_url()
        self.object.delete()
        return HttpResponseRedirect(success_url)

    def post(self, request, *args, **kwargs):
        return self.disapprove(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PostDisapproveView, self).get_context_data(**kwargs)
        context['forum'] = self.get_object().topic.forum
        return context

    def get_success_url(self):
        messages.success(self.request, self.success_message)
        return reverse('forum_moderation:queue')

    # Permissions checks

    def get_controlled_object(self):
        return self.get_object().topic.forum

    def perform_permissions_check(self, user, obj, perms):
        return self.request.forum_permission_handler.can_approve_posts(obj, user)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
import os

__version__ = '0.1.1'


MACHINA_VANILLA_APPS = [
    'machina',
    'machina.apps.forum',
    'machina.apps.forum_conversation',
    'machina.apps.forum_conversation.forum_attachments',
    'machina.apps.forum_conversation.forum_polls',
    'machina.apps.forum_feeds',
    'machina.apps.forum_moderation',
    'machina.apps.forum_search',
    'machina.apps.forum_tracking',
    'machina.apps.forum_member',
    'machina.apps.forum_permission',
]


# Main Machina static directory
MACHINA_MAIN_STATIC_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'static/machina/build')

# Main Machina template directory
MACHINA_MAIN_TEMPLATE_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'templates/machina')


def get_apps(overrides=None):
    """
    Returns a list of machina's apps. Any of these apps can be overriden with
    custom apps specified in the 'overrides' list.
    """
    if not overrides:
        return MACHINA_VANILLA_APPS

    def get_app_label(app_label):
        pattern = app_label.replace('machina.apps.', '')
        return next((o for o in overrides if o.endswith(pattern)), app_label)

    return list(map(get_app_label, MACHINA_VANILLA_APPS))



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.conf.urls import include
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class BoardApp(Application):
    name = None

    forum_app = get_class('forum.app', 'application')
    conversation_app = get_class('forum_conversation.app', 'application')
    feeds_app = get_class('forum_feeds.app', 'application')
    member_app = get_class('forum_member.app', 'application')
    moderation_app = get_class('forum_moderation.app', 'application')
    search_app = get_class('forum_search.app', 'application')
    tracking_app = get_class('forum_tracking.app', 'application')

    def get_urls(self):
        urls = [
            url(r'', include(self.forum_app.urls)),
            url(r'', include(self.conversation_app.urls)),
            url(_(r'^feeds/'), include(self.feeds_app.urls)),
            url(_(r'^member/'), include(self.member_app.urls)),
            url(_(r'^moderation/'), include(self.moderation_app.urls)),
            url(_(r'^search/'), include(self.search_app.urls)),
            url(_(r'^tracking/'), include(self.tracking_app.urls)),
        ]
        return urls


board = forum = application = BoardApp()



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.db import models

# Local application / specific library imports


class ApprovedManager(models.Manager):
    def get_queryset(self):
        """
        Returns all the approved topics or posts.
        """
        qs = super(ApprovedManager, self).get_queryset()
        qs = qs.filter(approved=True)

        return qs



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf import settings
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports


@python_2_unicode_compatible
class AbstractForumPermission(models.Model):
    """
    Represents a single forum permission.
    """
    codename = models.CharField(max_length=150, verbose_name=_('Permission codename'), unique=True)
    name = models.CharField(max_length=255, verbose_name=_('Permission name'), blank=True, null=True)

    is_global = models.BooleanField(
        verbose_name=_('Global permission'),
        help_text=_('This permission can be granted globally to all the forums'),
        default=False)
    is_local = models.BooleanField(
        verbose_name=_('Local permission'),
        help_text=_('This permission can be granted individually for each forum'),
        default=True)

    class Meta:
        abstract = True
        app_label = 'forum_permission'
        verbose_name = _('Forum permission')
        verbose_name_plural = _('Forum permissions')

    def __str__(self):
        if self.name:
            return '{} - {}'.format(self.codename, self.name)
        return self.codename

    def clean(self):
        super(AbstractForumPermission, self).clean()
        if not self.is_global and not self.is_local:
            raise ValidationError(_('A forum permission should be at least either global or local'))


class BaseAuthForumPermission(models.Model):
    """
    Represents a per-auth-component forum object permission.
    """
    permission = models.ForeignKey('forum_permission.ForumPermission', verbose_name=_('Forum permission'))
    has_perm = models.BooleanField(verbose_name=_('Has perm'), default=True)

    # The forum related to a UserForumPermission instance can be null if the
    # considered permission should be granted globally.
    forum = models.ForeignKey('forum.Forum', verbose_name=_('Forum'), blank=True, null=True)

    class Meta:
        abstract = True

    def clean(self):
        super(BaseAuthForumPermission, self).clean()
        if self.forum is None and not self.permission.is_global:
            raise ValidationError(
                _('The following permission cannot be granted globally: {}'.format(self.permission)))


@python_2_unicode_compatible
class AbstractUserForumPermission(BaseAuthForumPermission):
    """
    Represents a per-user forum object permission.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('User'), null=True, blank=True)
    anonymous_user = models.BooleanField(verbose_name=_('Target anonymous user'), default=False)

    class Meta:
        abstract = True
        unique_together = ('permission', 'forum', 'user', )
        app_label = 'forum_permission'
        verbose_name = _('User forum permission')
        verbose_name_plural = _('User forum permissions')

    def __str__(self):
        if self.forum:
            return '{} - {} - {}'.format(self.permission, self.user, self.forum)
        return '{} - {}'.format(self.permission, self.user)

    def clean(self):
        super(AbstractUserForumPermission, self).clean()
        if (self.user is None and not self.anonymous_user) \
                or (self.user and self.anonymous_user):
            raise ValidationError(_('A permission should target either a user or an anonymous user'))


@python_2_unicode_compatible
class AbstractGroupForumPermission(BaseAuthForumPermission):
    """
    Represents a per-group forum object permission.
    """
    group = models.ForeignKey(Group, verbose_name=_('Group'))

    class Meta:
        abstract = True
        unique_together = ('permission', 'forum', 'group', )
        app_label = 'forum_permission'
        verbose_name = _('Group forum permission')
        verbose_name_plural = _('Group forum permissions')

    def __str__(self):
        if self.forum:
            return '{} - {} - {}'.format(self.permission, self.group, self.forum)
        return '{} - {}'.format(self.permission, self.group)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.forms.widgets import Select
from django.utils.encoding import force_text
from django.utils.html import conditional_escape
from django.utils.html import escape

# Local application / specific library imports


# Originaly comes from https://djangosnippets.org/snippets/2453/
class SelectWithDisabled(Select):
    """
    Subclass of Django's select widget that allows disabling options.
    To disable an option, pass a dict instead of a string for its label,
    of the form: {'label': 'option label', 'disabled': True}
    """
    def render_option(self, selected_choices, option_value, option_label):
        option_value = force_text(option_value)
        if (option_value in selected_choices):
            selected_html = ' selected="selected"'
        else:
            selected_html = ''
        disabled_html = ''
        if isinstance(option_label, dict):
            if dict.get(option_label, 'disabled'):
                disabled_html = ' disabled="disabled"'
            option_label = option_label['label']
        return '<option value="%s"%s%s>%s</option>' % (
            escape(option_value), selected_html, disabled_html,
            conditional_escape(force_text(option_label)))



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports


class AttachmentsRegistryConfig(AppConfig):
    label = 'forum_attachments'
    name = 'machina.apps.forum_conversation.forum_attachments'
    verbose_name = _('Machina: Forum attachments')



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.conf import settings as machina_settings


def metadata(request):
    """
    Append some Machina-specific data to the template context.
    """
    return {
        'MACHINA_FORUM_NAME': machina_settings.MACHINA_FORUM_NAME,
    }



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPoll
from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPollOption
from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPollVote


class TopicPoll(AbstractTopicPoll):
    pass


class TopicPollOption(AbstractTopicPollOption):
    pass


class TopicPollVote(AbstractTopicPollVote):
    pass



# -*- coding: utf-8 -*-

# Standard library imports
import contextlib
# Third party imports
import mock

# Local application / specific library imports


# Originaly comes from mock-django (https://github.com/dcramer/mock-django)
@contextlib.contextmanager
def mock_signal_receiver(signal, wraps=None, **kwargs):
    """
    Temporarily attaches a receiver to the provided ``signal`` within the scope
    of the context manager.

    The mocked receiver is returned as the ``as`` target of the ``with``
    statement.

    To have the mocked receiver wrap a callable, pass the callable as the
    ``wraps`` keyword argument. All other keyword arguments provided are passed
    through to the signal's ``connect`` method.

    >>> with mock_signal_receiver(post_save, sender=Model) as receiver:
    >>>     Model.objects.create()
    >>>     assert receiver.call_count = 1
    """
    if wraps is None:
        wraps = lambda *args, **kwargs: None

    receiver = mock.Mock(wraps=wraps)
    signal.connect(receiver, **kwargs)
    yield receiver
    signal.disconnect(receiver)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.core import validators
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from machina.conf import settings as machina_settings
from machina.models.fields import ExtendedImageField
from machina.models.fields import MarkupTextField


@python_2_unicode_compatible
class AbstractForumProfile(models.Model):
    """
    Represents the profile associated with each forum user.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL, verbose_name=_('User'), related_name='forum_profile')

    # The user's avatar
    avatar = ExtendedImageField(
        verbose_name=_('Avatar'), null=True, blank=True,
        upload_to=machina_settings.PROFILE_AVATAR_UPLOAD_TO,
        **machina_settings.DEFAULT_AVATAR_SETTINGS)

    # The user's signature
    signature = MarkupTextField(
        verbose_name=_('Signature'), blank=True, null=True,
        validators=[validators.MaxLengthValidator(machina_settings.PROFILE_SIGNATURE_MAX_LENGTH)])

    # The amount of posts the user has posted
    posts_count = models.PositiveIntegerField(verbose_name=_('Total posts'), blank=True, default=0)

    class Meta:
        abstract = True
        app_label = 'forum_member'
        verbose_name = _('Forum profile')
        verbose_name_plural = _('Forum profiles')

    def __str__(self):
        return self.user.username



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from haystack import views

# Local application / specific library imports


class FacetedSearchView(views.FacetedSearchView):
    """
    Allows to search within forums
    """
    template = 'forum_search/search.html'

    def build_form(self):
        form = super(self.__class__, self).build_form(form_kwargs={'user': self.request.user})
        return form



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.db.models import F
from django.dispatch import receiver

# Local application / specific library imports
from machina.apps.forum_conversation.signals import topic_viewed


@receiver(topic_viewed)
def update_topic_counter(sender, topic, user, request, response, **kwargs):
    """
    Receiver to handle the update of the views counter associated with topics.
    """
    topic.__class__._default_manager.filter(id=topic.id).update(views_count=F('views_count') + 1)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_tracking.abstract_models import AbstractForumReadTrack
from machina.apps.forum_tracking.abstract_models import AbstractTopicReadTrack
from machina.core.db.models import model_factory


ForumReadTrack = model_factory(AbstractForumReadTrack)
TopicReadTrack = model_factory(AbstractTopicReadTrack)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class MemberApp(Application):
    name = 'forum_member'

    user_topics_view = get_class('forum_member.views', 'UserTopicsView')
    forum_profile_detail_view = get_class('forum_member.views', 'ForumProfileDetailView')
    forum_profile_update_view = get_class('forum_member.views', 'ForumProfileUpdateView')

    def get_urls(self):
        return [
            url(_(r'^profile/(?P<pk>\d+)/$'), self.forum_profile_detail_view.as_view(), name='profile'),
            url(_(r'^profile/edit/$'), self.forum_profile_update_view.as_view(), name='profile_update'),
            url(_(r'^ego/topics/$'), self.user_topics_view.as_view(), name='user_topics'),
        ]


application = MemberApp()



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.db.models import Q
from mptt.managers import TreeManager

# Local application / specific library imports


class ForumManager(TreeManager):
    def displayable_subforums(self, start_from=None):
        """
        Returns all the forums that can be seen from a forum at a given level in the tree
        of forums. A forum can be seen if one of the following statements is true:

            The forum is a direct child of the starting forum for the considered level.

            The forum have a parent which is a category and this category is a direct
            child of the starting forum.

            The forum have its 'display_sub_forum_list' option set to True and have a
            parent which is another forum. The latter is a direct child of the starting
            forum.

            The forum have its 'display_sub_forum_list' option set to True and have a
            parent which is another forum. The latter have a parent which is a category and
            this category is a direct child of the starting forum.

        If not starting forum is provided, the returned forums are those that can be seen from
        the root of the forums tree.
        """
        parent_field = 'isnull' if start_from is None else 'pk'
        parent_value = True if start_from is None else start_from.pk
        parent_selector = lambda x: '__'.join(['parent' for _ in range(0, x)] + [parent_field, ])

        return super(ForumManager, self).get_queryset().filter(
            # Forums that have a top-level category has parent
            Q(parent__type=self.model.TYPE_CHOICES.forum_cat, **{parent_selector(2): parent_value}) |
            # Sub forums that can be displayed
            Q(display_sub_forum_list=True, **{parent_selector(2): parent_value}) |
            # Children of forums that have a category as parent
            Q(parent__parent__type=self.model.TYPE_CHOICES.forum_cat,
                parent__type=self.model.TYPE_CHOICES.forum_post,
                display_sub_forum_list=True,
                **{parent_selector(3): parent_value}) |
            # Category, top-level forums and links
            Q(**{parent_selector(1): parent_value}))



# -*- coding: utf-8 -*-

# Standard library imports
import datetime as dt

# Third party imports
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.shortcuts import _get_queryset
from django.utils.timezone import now

# Local application / specific library imports
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from machina.core.loading import get_class

Forum = get_model('forum', 'Forum')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
Post = get_model('forum_conversation', 'Post')
TopicPollVote = get_model('forum_polls', 'TopicPollVote')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')

ForumPermissionChecker = get_class('forum_permission.checker', 'ForumPermissionChecker')

get_anonymous_user_forum_key = get_class('forum_permission.shortcuts', 'get_anonymous_user_forum_key')


class PermissionHandler(object):
    """
    The PermissionHandler allows to filter lists of forums and to perform
    permission verifications on forums. It uses the ForumPermissionChecker
    class to perform these verifications.
    """
    def __init__(self):
        # This dictionary will store the forums that are granted for a specific
        # lit of permission codenames and a given user.
        self._granted_forums_cache = {}

        # This one will store the ancestors of the forums that will be used to
        # perform permission checks.
        self._forum_ancestors_cache = {}

        # This one will store ForumPermissionChecker instances for users whose
        # permissions will be checked.
        self._user_perm_checkers_cache = {}

    # Filtering methods
    # --

    def forum_list_filter(self, qs, user):
        """
        Filters the given queryset in order to return a list of forums that can be seen or read
        by the specified user (at least).
        """
        # Any superuser should see all the forums
        if user.is_superuser:
            return qs

        # Check whether the forums can be viewed by the given user
        forums_to_hide = self._get_hidden_forum_ids(qs, user)

        return qs.exclude(id__in=forums_to_hide)

    def get_forum_last_post(self, forum, user):
        """
        Given a forum, fetch the last post that can be read by the passed user.
        """
        forums = forum.get_descendants(include_self=True)
        hidden_forums = []

        # Only non-superusers permissions are checked against the considered forums
        if not user.is_superuser:
            hidden_forums = self._get_hidden_forum_ids(forums, user)

        forums = forums.exclude(id__in=hidden_forums)
        posts = Post.approved_objects.filter(topic__forum__in=forums).order_by('-created')[:1]

        if not len(posts):
            return None
        return posts[0]

    # Verification methods
    # --

    # Forums

    def can_read_forum(self, forum, user):
        """
        Given a forum, checks whether the user can read its content.
        """
        return self._perform_basic_permission_check(forum, user, 'can_read_forum')

    # Posts and topics

    def can_add_topic(self, forum, user):
        """
        Given a forum, checks whether the user can append topics to it.
        """
        return self._perform_basic_permission_check(forum, user, 'can_start_new_topics')

    def can_add_stickies(self, forum, user):
        """
        Given a forum, checks whether the user can append stickies to it.
        """
        return self._perform_basic_permission_check(forum, user, 'can_post_stickies')

    def can_add_announcements(self, forum, user):
        """
        Given a forum, checks whether the user can append announcements to it.
        """
        return self._perform_basic_permission_check(forum, user, 'can_post_announcements')

    def can_post_without_approval(self, forum, user):
        """
        Given a forum, checks whether the user can add a posts and topics without approval.
        """
        return self._perform_basic_permission_check(forum, user, 'can_post_without_approval')

    def can_add_post(self, topic, user):
        """
        Given a topic, checks whether the user can append posts to it.
        """
        can_add_post = self._perform_basic_permission_check(topic.forum, user, 'can_reply_to_topics') \
            and (not topic.is_locked or
                 self._perform_basic_permission_check(topic.forum, user, 'can_reply_to_locked_topics'))
        return can_add_post

    def can_edit_post(self, post, user):
        """
        Given a forum post, checks whether the user can edit the latter.
        """
        checker = self._get_checker(user)

        # A user can edit a post if...
        #     he is a superuser
        #     he is the original poster of the forum post
        #     he belongs to the forum moderators
        is_author = self._is_post_author(post, user)
        can_edit = (user.is_superuser or
                    (is_author and checker.has_perm('can_edit_own_posts', post.topic.forum)) or
                    checker.has_perm('can_edit_posts', post.topic.forum))
        return can_edit

    def can_delete_post(self, post, user):
        """
        Given a forum post, checks whether the user can delete the latter.
        """
        checker = self._get_checker(user)

        # A user can delete a post if...
        #     he is a superuser
        #     he is the original poster of the forum post
        #     he belongs to the forum moderators
        is_author = self._is_post_author(post, user)
        can_delete = (user.is_superuser or
                      (is_author and checker.has_perm('can_delete_own_posts', post.topic.forum)) or
                      checker.has_perm('can_delete_posts', post.topic.forum))
        return can_delete

    # Polls

    def can_create_polls(self, forum, user):
        """
        Given a forum, checks whether the user can add a topic with an embedded poll.
        """
        return self._perform_basic_permission_check(forum, user, 'can_create_polls')

    def can_vote_in_poll(self, poll, user):
        """
        Given a poll, checks whether the user can answer to it.
        """
        # First we have to check if the poll is curently open
        if poll.duration:
            poll_dtend = poll.created + dt.timedelta(days=poll.duration)
            if poll_dtend < now():
                return False

        # Is this user allowed to vote in polls in the current forum ?
        can_vote = self._perform_basic_permission_check(poll.topic.forum, user, 'can_vote_in_polls') \
            and not poll.topic.is_locked

        # Retrieve the user votes for the considered poll
        user_votes = TopicPollVote.objects.filter(
            poll_option__poll=poll)
        if user.is_anonymous():
            forum_key = get_anonymous_user_forum_key(user)
            if forum_key:
                user_votes = user_votes.filter(anonymous_key=forum_key)
            else:
                # If the forum key of the anonymous user cannot be retrieved, the user
                # should not be allowed to vote in the considered poll.
                user_votes = user_votes.none()
                can_vote = False
        else:
            user_votes = user_votes.filter(voter=user)

        # If the user has already voted, he can vote again if the vote changes are allowed
        if user_votes.exists() and can_vote:
            can_vote = poll.user_changes

        return can_vote

    # Attachments

    def can_attach_files(self, forum, user):
        """
        Given a forum, checks whether the user can add attachments to posts.
        """
        return self._perform_basic_permission_check(forum, user, 'can_attach_file')

    def can_download_files(self, forum, user):
        """
        Given a forum, checks whether the user can download files attached to posts.
        """
        return self._perform_basic_permission_check(forum, user, 'can_download_file')

    # Moderation

    def get_moderation_queue_forums(self, user):
        """
        Returns the list of forums whose posts can be approved by the considered
        user.
        """
        return self._get_forums_for_user(user, ['can_approve_posts', ])

    def can_access_moderation_queue(self, user):
        """
        Returns True if the passed user can access the moderation queue.  The latest
        allows the moderator to approve posts.
        """
        moderated_forums = self.get_moderation_queue_forums(user)
        return moderated_forums.exists()

    def can_lock_topics(self, forum, user):
        """
        Given a forum, checks whether the user can lock its topics.
        """
        return self._perform_basic_permission_check(forum, user, 'can_lock_topics')

    def can_move_topics(self, forum, user):
        """
        Given a forum, checks whether the user can move its topics to another forum.
        """
        return self._perform_basic_permission_check(forum, user, 'can_move_topics')

    def get_target_forums_for_moved_topics(self, user):
        """
        Returns a list of forums in which the considered user can add topics
        that have been moved from another forum.
        """
        return self._get_forums_for_user(user, ['can_move_topics', ]) \
            .filter(type=Forum.TYPE_CHOICES.forum_post)

    def can_delete_topics(self, forum, user):
        """
        Given a forum, checks whether the user can delete its topics.
        Note: the ``can_delete_posts`` permission is used here because
        a user who can delete all the posts of a topic is also able to
        delete the topic itself.
        """
        return self._perform_basic_permission_check(forum, user, 'can_delete_posts')

    def can_update_topics_to_normal_topics(self, forum, user):
        """
        Given a forum, checks whether the user can change its topic types to normal
        topics.
        """
        return self._perform_basic_permission_check(forum, user, 'can_edit_posts')

    def can_update_topics_to_sticky_topics(self, forum, user):
        """
        Given a forum, checks whether the user can change its topic types to sticky
        topics.
        """
        return self._perform_basic_permission_check(forum, user, 'can_edit_posts') \
            and self._perform_basic_permission_check(forum, user, 'can_post_stickies')

    def can_update_topics_to_announces(self, forum, user):
        """
        Given a forum, checks whether the user can change its topic types to announces.
        """
        return self._perform_basic_permission_check(forum, user, 'can_edit_posts') \
            and self._perform_basic_permission_check(forum, user, 'can_post_announcements')

    def can_approve_posts(self, forum, user):
        """
        Given a forum, checks whether the user can approve its posts.
        """
        return self._perform_basic_permission_check(forum, user, 'can_approve_posts')

    # Common
    # --

    def _is_post_author(self, post, user):
        return (post.poster == user) if user.is_authenticated() \
            else (post.anonymous_key is not None and
                  post.anonymous_key == get_anonymous_user_forum_key(user))

    def _get_hidden_forum_ids(self, forums, user):
        """
        Given a set of forums and a user, returns the list of forums that are not visible
        by this user.
        """
        visible_forums = self._get_forums_for_user(user, ['can_see_forum', 'can_read_forum', ])
        hidden_forums = []

        for forum in forums:
            if forum.id not in hidden_forums:
                # First cheks if any of the forum ancestors is hidden
                forum_ancestor_ids = set(self._get_forum_ancestors_ids(forum))
                ancestors_visible = forum_ancestor_ids.issubset(set(f.id for f in visible_forums)) \
                    if forum.parent_id else True

                if (ancestors_visible is False) or (forum not in visible_forums):
                    # If one forum can not be seen by a given user, all of its descendants
                    # should also be hidden.
                    forum_and_descendants = forum.get_descendants(include_self=True)
                    hidden_forums.extend(forum_and_descendants.values_list('id', flat=True))

        return hidden_forums

    def _get_forum_ancestors_ids(self, forum):
        """
        Returns the ancestor IDs of the given forum. These parent forum identifiers are
        stored inside a local cache for further use.
        """
        forum_ancestors_cache_key = '{}__ancestors'.format(forum.pk)

        if forum_ancestors_cache_key in self._forum_ancestors_cache:
            return self._forum_ancestors_cache[forum_ancestors_cache_key]

        forum_ancestor_ids = list(forum.get_ancestors().values_list('id', flat=True))
        self._forum_ancestors_cache[forum_ancestors_cache_key] = forum_ancestor_ids
        return forum_ancestor_ids

    def _get_forums_for_user(self, user, perm_codenames):
        """
        Returns all the forums that satisfy the given list of permission
        codenames. User and group forum permissions are used.
        """
        granted_forums_cache_key = '{}__{}'.format(
            ':'.join(perm_codenames),
            user.id if not user.is_anonymous() else 'anonymous')

        if granted_forums_cache_key in self._granted_forums_cache:
            return self._granted_forums_cache[granted_forums_cache_key]

        forum_queryset = _get_queryset(Forum)

        # First check if the user is a superuser and if so, returns the forum
        # queryset immediately.
        if user.is_superuser:  # pragma: no cover
            forum_objects = forum_queryset

        else:
            # Generates the appropriate queryset filter in order to handle both
            # authenticated users and anonymous users.
            user_kwargs_filter = {'anonymous_user': True} if user.is_anonymous() \
                else {'user': user}

            # Get all the user permissions for the considered user.
            user_perms = UserForumPermission.objects \
                .filter(**user_kwargs_filter) \
                .filter(permission__codename__in=perm_codenames)

            globally_granted_user_perms = list(filter(lambda p: p.has_perm and p.forum is None, user_perms))
            per_forum_granted_user_perms = list(filter(lambda p: p.has_perm and p.forum is not None, user_perms))
            per_forum_nongranted_user_perms = list(filter(lambda p: not p.has_perm and p.forum is not None, user_perms))

            user_granted_forum_ids = [p.forum_id for p in per_forum_granted_user_perms]
            user_nongranted_forum_ids = [p.forum_id for p in per_forum_nongranted_user_perms]

            if not user.is_anonymous():
                # Get all the group permissions for the considered user.
                group_perms = GroupForumPermission.objects \
                    .filter(**{'group__{0}'.format(get_user_model().groups.field.related_query_name()): user}) \
                    .filter(permission__codename__in=perm_codenames)

                globally_granted_group_perms = list(filter(lambda p: p.has_perm and p.forum is None, group_perms))
                per_forum_granted_group_perms = list(filter(lambda p: p.has_perm and p.forum is not None, group_perms))
                per_forum_nongranted_group_perms = list(filter(lambda p: not p.has_perm and p.forum is not None, group_perms))

                group_granted_forum_ids = [p.forum_id for p in per_forum_granted_group_perms]
                group_nongranted_forum_ids = [p.forum_id for p in per_forum_nongranted_group_perms]

                if globally_granted_user_perms or globally_granted_group_perms:
                    forum_objects = forum_queryset.filter(
                        ~Q(pk__in=(user_nongranted_forum_ids + group_nongranted_forum_ids)))
                else:
                    forum_objects = forum_queryset.filter(
                        Q(pk__in=user_granted_forum_ids) | Q(pk__in=group_granted_forum_ids)) \
                        .filter(~Q(pk__in=(user_nongranted_forum_ids + group_nongranted_forum_ids)))
            else:
                if globally_granted_user_perms:
                    forum_objects = forum_queryset.filter(~Q(pk__in=user_nongranted_forum_ids))
                else:
                    forum_objects = forum_queryset.filter(Q(pk__in=user_granted_forum_ids)) \
                        .filter(~Q(pk__in=user_nongranted_forum_ids))

            if not user.is_anonymous() and not forum_objects.exists() and set(perm_codenames).issubset(set(
                    machina_settings.DEFAULT_AUTHENTICATED_USER_FORUM_PERMISSIONS)):
                forum_objects = forum_queryset

        self._granted_forums_cache[granted_forums_cache_key] = forum_objects
        return forum_objects

    def _perform_basic_permission_check(self, forum, user, permission):
        """
        Given a forum and a user, checks whether the latter has the passed
        permission.
        The workflow is:

            1. The permission is granted if the user is a superuser
            2. If not, a check is performed with the given permission
        """
        checker = self._get_checker(user)

        # The action is granted if...
        #     the user is the superuser
        #     the user has the permission to do so
        check = (user.is_superuser or checker.has_perm(permission, forum))
        return check

    def _get_checker(self, user):
        """
        Return a ForumPermissionChecker instance for the given user.
        """
        user_perm_checkers_cache_key = user.id if not user.is_anonymous() else 'anonymous'

        if user_perm_checkers_cache_key in self._user_perm_checkers_cache:
            return self._user_perm_checkers_cache[user_perm_checkers_cache_key]

        checker = ForumPermissionChecker(user)
        self._user_perm_checkers_cache[user_perm_checkers_cache_key] = checker
        return checker



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django import template

# Local application / specific library imports
from machina.core.loading import get_class

TrackingHandler = get_class('forum_tracking.handler', 'TrackingHandler')

register = template.Library()


@register.assignment_tag(takes_context=True)
def get_unread_forums(context, forums, user):
    """
    This will return a list of unread forums for the given user from a given set of forums.

    Usage::

        {% get_unread_forums forums request.user as unread_forums %}
    """
    request = context.get('request', None)
    return TrackingHandler(request=request).get_unread_forums(forums, user)


@register.assignment_tag(takes_context=True)
def get_unread_topics(context, topics, user):
    """
    This will return a list of unread topics for the given user from a given set of forums.

    Usage::

        {% get_unread_topics topics request.user as unread_topics %}
    """
    request = context.get('request', None)
    return TrackingHandler(request=request).get_unread_topics(topics, user)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_conversation.forum_attachments.abstract_models import AbstractAttachment


class Attachment(AbstractAttachment):
    pass



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports


def get_client_ip(request):
    """
    Given an HTTP request, returns the related IP address.
    """
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR', None)
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.db import models

# Local application / specific library imports


class ForumReadTrackManager(models.Manager):
    def get_unread_forums_from_list(self, forums, user):
        """
        Given a list of forums find and returns the list of forums that are unread
        for the passed user.
        If a forum is unread all of its ancestors are also unread and will be
        included in the final list.
        """
        unread_forums = []

        tracks = super(ForumReadTrackManager, self).get_queryset().select_related('forum').filter(
            user=user,
            forum__in=forums)
        tracked_forums = []

        for track in tracks:
            if (track.forum.last_post_on and track.mark_time < track.forum.last_post_on) \
                    and track.forum not in unread_forums:
                unread_forums.extend(track.forum.get_ancestors(include_self=True))
            tracked_forums.append(track.forum)

        for forum in forums:
            if forum not in tracked_forums and forum not in unread_forums \
                    and forum.topics.filter(approved=True).count() > 0:
                unread_forums.extend(forum.get_ancestors(include_self=True))

        return list(set(unread_forums))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import machina.models.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ForumProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('avatar', machina.models.fields.ExtendedImageField(upload_to='machina/avatar_images', null=True, verbose_name='Avatar', blank=True)),
                ('signature', machina.models.fields.MarkupTextField(max_length=255, no_rendered_field=True, null=True, verbose_name='Signature', blank=True)),
                ('posts_count', models.PositiveIntegerField(default=0, verbose_name='Total posts', blank=True)),
                ('_signature_rendered', models.TextField(null=True, editable=False, blank=True)),
                ('user', models.OneToOneField(related_name='forum_profile', verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Forum profile',
                'verbose_name_plural': 'Forum profiles',
            },
        ),
    ]



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView
from django.views.generic import View

# Local application / specific library imports
from machina.conf import settings as machina_settings
from machina.core.db.models import get_model
from machina.core.loading import get_class
from machina.core.loading import get_classes

Forum = get_model('forum', 'Forum')
ForumReadTrack, TopicReadTrack = get_classes('forum_tracking.models',
                                             ['ForumReadTrack', 'TopicReadTrack'])
Topic = get_model('forum_conversation', 'Topic')

TrackingHandler = get_class('forum_tracking.handler', 'TrackingHandler')
track_handler = TrackingHandler()

PermissionRequiredMixin = get_class('forum_permission.viewmixins', 'PermissionRequiredMixin')


class MarkForumsReadView(View):
    """
    Marks a set of forums as read.
    """
    success_message = _('Forums have been marked read.')

    def get(self, request, pk=None):
        top_level_forum = None

        if pk is not None:
            top_level_forum = get_object_or_404(Forum, pk=pk)
            forums = request.forum_permission_handler.forum_list_filter(
                top_level_forum.get_descendants(include_self=True), request.user)
            redirect_to = reverse('forum:forum', kwargs={'slug': top_level_forum.slug, 'pk': pk})
        else:
            forums = request.forum_permission_handler.forum_list_filter(
                Forum.objects.all(), request.user)
            redirect_to = reverse('forum:index')

        # Marks forums as read
        track_handler.mark_forums_read(forums, request.user)

        if len(forums):
            messages.success(request, self.success_message)

        return HttpResponseRedirect(redirect_to)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(MarkForumsReadView, self).dispatch(request, *args, **kwargs)


class MarkTopicsReadView(PermissionRequiredMixin, View):
    """
    Marks a set of topics as read.
    """
    success_message = _('Topics have been marked read.')
    permission_required = ['can_read_forum', ]

    def get(self, request, pk):
        forum = get_object_or_404(Forum, pk=pk)

        # Marks forum topics as read
        track_handler.mark_forums_read([forum, ], request.user)

        messages.success(request, self.success_message)
        redirect_to = reverse('forum:forum', kwargs={'slug': forum.slug, 'pk': pk})

        return HttpResponseRedirect(redirect_to)

    def get_controlled_object(self):
        """
        Return the considered forum in order to allow permission checks.
        """
        return Forum.objects.get(pk=self.kwargs['pk'])

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(MarkTopicsReadView, self).dispatch(request, *args, **kwargs)


class UnreadTopicsView(ListView):
    """
    Displays unread topics for the current user.
    """
    template_name = 'forum_tracking/unread_topic_list.html'
    context_object_name = 'topics'
    paginate_by = machina_settings.FORUM_TOPICS_NUMBER_PER_PAGE

    def get_queryset(self):
        forums = self.request.forum_permission_handler.forum_list_filter(
            Forum.objects.all(), self.request.user)
        topics = Topic.objects.filter(forum__in=forums)
        topics_pk = map(lambda t: t.pk, track_handler.get_unread_topics(topics, self.request.user))
        return Topic.approved_objects.filter(pk__in=topics_pk).order_by('-last_post_on')

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(UnreadTopicsView, self).dispatch(request, *args, **kwargs)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_permission.abstract_models import AbstractForumPermission
from machina.apps.forum_permission.abstract_models import AbstractGroupForumPermission
from machina.apps.forum_permission.abstract_models import AbstractUserForumPermission
from machina.core.db.models import model_factory


ForumPermission = model_factory(AbstractForumPermission)
GroupForumPermission = model_factory(AbstractGroupForumPermission)
UserForumPermission = model_factory(AbstractUserForumPermission)



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf import settings
from django.core.cache import caches
from django.core.cache import InvalidCacheBackendError
from django.core.exceptions import ImproperlyConfigured
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.core.files.uploadedfile import TemporaryUploadedFile
from django.utils.datastructures import MultiValueDict
from django.utils.six import BytesIO

# Local application / specific library imports
from machina.conf import settings as machina_settings


class AttachmentCache(object):
    """
    The attachments cache. This one should be used with a FileBasedCache backend.
    But this can be overriden. The attachments cache acts as a wrapper and ensure
    that the states (name, size, content type, charset and content) of all files
    from any request.FILES dict are saved inside the considered backend when calling
    the 'set' method. Conversely, the 'get' method will populate a dictionary of
    InMemoryUploadedFile instances or TemporaryUploadedFile instancesby using these
    states.
    """
    def __init__(self):
        self.backend = self.get_backend()

    def get_backend(self):
        try:
            cache = caches[machina_settings.ATTACHMENT_CACHE_NAME]
        except InvalidCacheBackendError:
            raise ImproperlyConfigured(
                'The attachment cache backend ({}) is not configured'.format(machina_settings.ATTACHMENT_CACHE_NAME))
        return cache

    def set(self, key, files):
        """
        Stores the state of each file embedded in the request.FILES MultiValueDict
        instance. This instance is assumed to be passed as the 'files' argument.
        Each state stored in the cache is a dictionary containing the following
        values:

            name
                The name of the uploaded file.
            size
                The size of the uploaded file.
            content_type
                The content type of the uploaded file.
            content_length
                The content length of the uploaded file.
            charset
                The charset of the uploaded file.
            content
                The content of the uploaded file.
        """
        files_states = {}

        for name, upload in files.items():
            # Generates the state of the file
            state = {
                'name': upload.name,
                'size': upload.size,
                'content_type': upload.content_type,
                'charset': upload.charset,
                'content': upload.file.read(),
            }
            files_states[name] = state

            # Go to the first byte in the file for future use
            upload.file.seek(0)
        self.backend.set(key, files_states)

    def get(self, key):
        """
        Regenerates a MultiValueDict instance containing the files related to all
        file states stored for the given key.
        """
        upload = None
        files_states = self.backend.get(key)
        files = MultiValueDict()
        if files_states:
            for name, state in files_states.items():
                f = BytesIO()
                f.write(state['content'])

                # If the post is too large, we cannot use a
                # InMemoryUploadedFile instance.
                if state['size'] > settings.FILE_UPLOAD_MAX_MEMORY_SIZE:
                    upload = TemporaryUploadedFile(
                        state['name'],
                        state['content_type'],
                        state['size'],
                        state['charset'])
                    upload.file = f
                else:
                    f = BytesIO()
                    f.write(state['content'])
                    upload = InMemoryUploadedFile(
                        file=f,
                        field_name=name,
                        name=state['name'],
                        content_type=state['content_type'],
                        size=state['size'],
                        charset=state['charset'])
                files[name] = upload

                # Go to the first byte in the file for future use
                upload.file.seek(0)

        return files

    def delete(self, key):
        self.backend.delete(key)


cache = AttachmentCache()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

# Local application / specific library imports
from machina.core.app import Application
from machina.core.loading import get_class


class FeedsApp(Application):
    name = 'forum_feeds'

    latest_topics_feed = get_class('forum_feeds.feeds', 'LastTopicsFeed')

    def get_urls(self):
        return [
            url(_(r'^topics/$'), self.latest_topics_feed(), name='latest_topics'),
            url(_(r'^forum/(?P<forum_slug>[\w-]+)-(?P<forum_pk>\d+)/topics/$'),
                self.latest_topics_feed(), name='forum_latest_topics'),
            url(_(r'^forum/(?P<forum_slug>[\w-]+)-(?P<forum_pk>\d+)/topics/all/$'),
                self.latest_topics_feed(), {'descendants': True}, name='forum_latest_topics_with_descendants'),
        ]


application = FeedsApp()



# -*- coding: utf-8 -*-

# Standard library imports
from __future__ import unicode_literals

# Third party imports
import factory
from faker import Factory as FakerFactory

# Local application / specific library imports
from machina.core.db.models import get_model

faker = FakerFactory.create()

ForumPermission = get_model('forum_permission', 'ForumPermission')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')


class ForumPermissionFactory(factory.DjangoModelFactory):
    codename = factory.Sequence(lambda n: 'perm-{}'.format(n))
    name = faker.text(max_nb_chars=150)

    class Meta:
        model = ForumPermission


class UserForumPermissionFactory(factory.DjangoModelFactory):
    permission = factory.SubFactory(ForumPermissionFactory)

    class Meta:
        model = UserForumPermission


class GroupForumPermissionFactory(factory.DjangoModelFactory):
    permission = factory.SubFactory(ForumPermissionFactory)

    class Meta:
        model = GroupForumPermission



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import F
from django.db.models.signals import pre_save
from django.db.models.signals import post_delete
from django.dispatch import receiver

from machina.core.loading import get_class

User = get_user_model()

Post = get_class('forum_conversation.models', 'Post')
ForumProfile = get_class('forum_member.models', 'ForumProfile')


@receiver(pre_save, sender=Post)
def increase_posts_count(sender, instance, **kwargs):
    """
    Receiver to handle the update of the profile related to the user
    who is the poster of the forum post being created or updated.
    """
    if instance.poster is None:
        # An anonymous post is considered. No profile can be updated in
        # that case.
        return

    profile, dummy = ForumProfile.objects.get_or_create(user=instance.poster)
    increase_posts_count = False

    if instance.pk:
        try:
            old_instance = instance.__class__._default_manager.get(pk=instance.pk)
        except ObjectDoesNotExist:  # pragma: no cover
            # This should never happen (except with django loaddata command)
            increase_posts_count = True
            old_instance = None
        if old_instance and old_instance.approved is False and instance.approved is True:
            increase_posts_count = True
    elif instance.approved:
        increase_posts_count = True

    if increase_posts_count:
        profile.posts_count = F('posts_count') + 1
        profile.save()


@receiver(post_delete, sender=Post)
def decrease_posts_count(sender, instance, **kwargs):
    """
    Receiver to handle the deletion of a forum posts: the posts count
    related to the post's author is decreased.
    """
    try:
        assert instance.poster_id is not None
        poster = User.objects.get(pk=instance.poster_id)
    except AssertionError:
        # An anonymous post is considered. No profile can be updated in
        # that case.
        return
    except ObjectDoesNotExist:  # pragma: no cover
        # This can happen if a User instance is deleted. In that case the
        # User instance is not available and the receiver should return.
        return

    profile, dummy = ForumProfile.objects.get_or_create(user=poster)
    profile.posts_count = F('posts_count') - 1
    profile.save()



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_member.abstract_models import AbstractForumProfile
from machina.core.db.models import model_factory


ForumProfile = model_factory(AbstractForumProfile)



# -*- coding: utf-8 -*-

# Standard library imports
# Third party imports
# Local application / specific library imports
from machina.apps.forum_conversation.abstract_models import AbstractPost
from machina.apps.forum_conversation.abstract_models import AbstractTopic
from machina.core.db.models import model_factory


Topic = model_factory(AbstractTopic)
Post = model_factory(AbstractPost)
